@@ -520,6 +520,8 @@ std::string AttachPoint::name(const std::string &attach_target,
   std::string n = provider;
   if (attach_target != "")
     n += ":" + attach_target;
+  if (lang != "")
+    n += ":" + lang;
   if (ns != "")
     n += ":" + ns;
   if (attach_point != "")
@@ -580,6 +582,11 @@ std::string Probe::name() const
   return n;
 }
 
+std::string Probe::args_typename() const
+{
+  return "struct " + name() + "_args";
+}
+
 int Probe::index() const
 {
   return index_;
