@@ -559,6 +559,7 @@ public:
 
   std::string provider;
   std::string target;
+  std::string lang; // for userspace probes, enable language-specific features
   std::string ns;
   std::string func;
   std::string pin;
@@ -599,6 +600,7 @@ public:
   StatementList *stmts = nullptr;
 
   std::string name() const;
+  std::string args_typename() const;
   bool need_expansion = false;        // must build a BPF program per wildcard match
   int tp_args_structs_level = -1;     // number of levels of structs that must
                                       // be imported/resolved for tracepoints
