@@ -23,7 +23,7 @@ std::optional<uint64_t> AttachPointParser::stoull(const std::string &str)
   }
   catch (const std::exception &e)
   {
-    errs_ << e.what();
+    errs_ << e.what() << std::endl;
     return std::nullopt;
   }
 }
@@ -32,11 +32,11 @@ std::optional<int64_t> AttachPointParser::stoll(const std::string &str)
 {
   try
   {
-    return int_parser::to_uint(str, 0);
+    return int_parser::to_int(str, 0);
   }
   catch (const std::exception &e)
   {
-    errs_ << e.what();
+    errs_ << e.what() << std::endl;
     return std::nullopt;
   }
 }
@@ -67,7 +67,6 @@ int AttachPointParser::parse()
       {
         ++failed;
         LOG(ERROR, ap.loc, sink_) << errs_.str();
-        errs_.str({}); // clear buffer
       }
       else if (s == SKIP || s == NEW_APS)
       {
@@ -82,6 +81,10 @@ int AttachPointParser::parse()
                                        new_attach_points.end());
         }
       }
+
+      // clear error buffer between attach points to prevent non-fatal errors
+      // from being carried over and printed on the next fatal error
+      errs_.str({});
     }
 
     auto new_end = std::remove_if(probe->attach_points->begin(),
@@ -341,14 +344,12 @@ AttachPointParser::State AttachPointParser::kprobe_parser(bool allow_offset)
     ap_->func = offset_parts[0];
 
     auto res = stoll(offset_parts[1]);
-    if (res)
-    {
-      ap_->func_offset = *res;
-    }
-    else
+    if (!res)
     {
+      errs_ << "Invalid offset" << std::endl;
       return INVALID;
     }
+    ap_->func_offset = *res;
   }
   // Default case (eg kprobe:func)
   else
@@ -370,22 +371,31 @@ AttachPointParser::State AttachPointParser::kretprobe_parser()
 AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
                                                           bool allow_abs_addr)
 {
-  if (parts_.size() == 2 && bpftrace_.pid() > 0)
+  if (bpftrace_.pid() > 0 &&
+      (parts_.size() == 2 ||
+       (parts_.size() == 3 && is_supported_lang(parts_[1]))))
   {
     // For PID, the target may be skipped
-    parts_.push_back(parts_[1]);
+    if (parts_.size() == 2)
+      parts_.insert(parts_.begin() + 1, "");
+
     auto target = get_pid_exe(bpftrace_.pid());
     parts_[1] = path_for_pid_mountns(bpftrace_.pid(), target);
   }
-  if (parts_.size() != 3)
+
+  if (parts_.size() != 3 && parts_.size() != 4)
   {
     if (ap_->ignore_invalid)
       return SKIP;
 
-    errs_ << ap_->provider << " probe type requires 2 arguments" << std::endl;
+    errs_ << ap_->provider << " probe type requires 2 or 3 arguments"
+          << std::endl;
     return INVALID;
   }
 
+  if (parts_.size() == 4)
+    ap_->lang = parts_[2];
+
   ap_->target = "";
 
   if (!has_wildcard(parts_[1]) && parts_[1].find("lib") == 0)
@@ -405,8 +415,9 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
     ap_->target = parts_[1];
   }
 
+  const std::string &func = parts_.back();
   // Handle uprobe:/lib/asdf:func+0x100 case
-  auto plus_count = std::count(parts_[2].cbegin(), parts_[2].cend(), '+');
+  auto plus_count = std::count(func.cbegin(), func.cend(), '+');
   if (plus_count)
   {
     if (!allow_offset)
@@ -421,7 +432,7 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
       return INVALID;
     }
 
-    auto offset_parts = split_string(parts_[2], '+', true);
+    auto offset_parts = split_string(func, '+', true);
     if (offset_parts.size() != 2)
     {
       errs_ << "Invalid offset" << std::endl;
@@ -431,21 +442,19 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
     ap_->func = offset_parts[0];
 
     auto res = stoll(offset_parts[1]);
-    if (res)
-    {
-      ap_->func_offset = *res;
-    }
-    else
+    if (!res)
     {
+      errs_ << "Invalid offset" << std::endl;
       return INVALID;
     }
+    ap_->func_offset = *res;
   }
   // Default case (eg uprobe:[addr][func])
   else
   {
     if (allow_abs_addr)
     {
-      auto res = stoll(parts_[2]);
+      auto res = stoll(func);
       if (res)
       {
         if (has_wildcard(ap_->target))
@@ -458,11 +467,11 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
       else
       {
         ap_->address = 0;
-        ap_->func = parts_[2];
+        ap_->func = func;
       }
     }
     else
-      ap_->func = parts_[2];
+      ap_->func = func;
   }
 
   if (ap_->target.find('*') != std::string::npos ||
@@ -547,6 +556,14 @@ AttachPointParser::State AttachPointParser::tracepoint_parser()
 
 AttachPointParser::State AttachPointParser::profile_parser()
 {
+  if (parts_.size() == 2 && has_wildcard(parts_[1]))
+  {
+    // Wildcards are allowed for listing
+    ap_->target = parts_[1];
+    ap_->freq = 0;
+    return OK;
+  }
+
   if (parts_.size() != 3)
   {
     errs_ << ap_->provider << " probe type requires 2 arguments" << std::endl;
@@ -556,16 +573,26 @@ AttachPointParser::State AttachPointParser::profile_parser()
   ap_->target = parts_[1];
 
   auto res = stoull(parts_[2]);
-  if (res)
-    ap_->freq = *res;
-  else
+  if (!res)
+  {
+    errs_ << "Invalid rate of " << ap_->provider << " probe";
     return INVALID;
+  }
 
+  ap_->freq = *res;
   return OK;
 }
 
 AttachPointParser::State AttachPointParser::interval_parser()
 {
+  if (parts_.size() == 2 && has_wildcard(parts_[1]))
+  {
+    // Wildcards are allowed for listing
+    ap_->target = parts_[1];
+    ap_->freq = 0;
+    return OK;
+  }
+
   if (parts_.size() != 3)
   {
     errs_ << ap_->provider << " probe type requires 2 arguments" << std::endl;
@@ -574,11 +601,13 @@ AttachPointParser::State AttachPointParser::interval_parser()
 
   ap_->target = parts_[1];
   auto res = stoull(parts_[2]);
-  if (res)
-    ap_->freq = *res;
-  else
+  if (!res)
+  {
+    errs_ << "Invalid rate of " << ap_->provider << " probe";
     return INVALID;
+  }
 
+  ap_->freq = *res;
   return OK;
 }
 
@@ -599,10 +628,12 @@ AttachPointParser::State AttachPointParser::software_parser()
   if (parts_.size() == 3 && parts_[2] != "*")
   {
     auto res = stoull(parts_[2]);
-    if (res)
-      ap_->freq = *res;
-    else
+    if (!res)
+    {
+      errs_ << "Invalid count for " << ap_->provider << " probe";
       return INVALID;
+    }
+    ap_->freq = *res;
   }
 
   return OK;
@@ -625,10 +656,12 @@ AttachPointParser::State AttachPointParser::hardware_parser()
   if (parts_.size() == 3 && parts_[2] != "*")
   {
     auto res = stoull(parts_[2]);
-    if (res)
-      ap_->freq = *res;
-    else
+    if (!res)
+    {
+      errs_ << "Invalid count for " << ap_->provider << " probe";
       return INVALID;
+    }
+    ap_->freq = *res;
   }
 
   return OK;
@@ -646,10 +679,12 @@ AttachPointParser::State AttachPointParser::watchpoint_parser(bool async)
   if (parts_[1].find('+') == std::string::npos)
   {
     auto parsed = stoull(parts_[1]);
-    if (parsed)
-      ap_->address = *parsed;
-    else
+    if (!parsed)
+    {
+      errs_ << "Invalid function/address argument" << std::endl;
       return INVALID;
+    }
+    ap_->address = *parsed;
   }
   else
   {
@@ -671,17 +706,21 @@ AttachPointParser::State AttachPointParser::watchpoint_parser(bool async)
     }
 
     auto parsed = stoull(func_arg_parts[1].substr(3));
-    if (parsed)
-      ap_->address = *parsed;
-    else
+    if (!parsed)
+    {
+      errs_ << "Invalid function argument" << std::endl;
       return INVALID;
+    }
+    ap_->address = *parsed;
   }
 
   auto len_parsed = stoull(parts_[2]);
-  if (len_parsed)
-    ap_->len = *len_parsed;
-  else
+  if (!len_parsed)
+  {
+    errs_ << "Invalid length argument" << std::endl;
     return INVALID;
+  }
+  ap_->len = *len_parsed;
 
   // Semantic analyser will ensure a cmd/pid was provided
   ap_->target = bpftrace_.get_watchpoint_binary_path().value_or("");
@@ -795,13 +834,7 @@ AttachPointParser::State AttachPointParser::raw_tracepoint_parser()
   ap_->func = parts_[1];
 
   if (has_wildcard(ap_->func))
-  {
-    if (ap_->ignore_invalid)
-      return SKIP;
-    errs_ << ap_->provider << " probe type does not support wildcards"
-          << std::endl;
-    return INVALID;
-  }
+    ap_->need_expansion = true;
 
   return OK;
 }
