@@ -4,7 +4,7 @@
 #include <sstream>
 #include <vector>
 
-#include "ast.h"
+#include "ast/ast.h"
 #include "bpftrace.h"
 
 namespace bpftrace {
@@ -44,6 +44,7 @@ private:
    */
   State lex_attachpoint(const AttachPoint &ap);
 
+  State special_parser();
   State kprobe_parser(bool allow_offset = true);
   State kretprobe_parser();
   State uprobe_parser(bool allow_offset = true, bool allow_abs_addr = true);
@@ -57,6 +58,7 @@ private:
   State watchpoint_parser(bool async = false);
   State kfunc_parser();
   State iter_parser();
+  State raw_tracepoint_parser();
 
   std::optional<uint64_t> stoull(const std::string &str);
   std::optional<int64_t> stoll(const std::string &str);
