@@ -82,23 +82,30 @@ libbpf::bpf_func_id IRBuilderBPF::selectProbeReadHelper(AddrSpace as, bool str)
   return fn;
 }
 
-AllocaInst *IRBuilderBPF::CreateUSym(llvm::Value *val, const location &loc)
+AllocaInst *IRBuilderBPF::CreateUSym(llvm::Value *val,
+                                     int probe_id,
+                                     const location &loc)
 {
   std::vector<llvm::Type *> elements = {
     getInt64Ty(), // addr
     getInt64Ty(), // pid
+    getInt64Ty(), // probe id
   };
   StructType *usym_t = GetStructType("usym_t", elements, false);
   AllocaInst *buf = CreateAllocaBPF(usym_t, "usym");
 
   Value *pid = CreateLShr(CreateGetPidTgid(loc), 32);
+  Value *probe_id_val = Constant::getIntegerValue(getInt64Ty(),
+                                                  APInt(64, probe_id));
 
   // The extra 0 here ensures the type of addr_offset will be int64
   Value *addr_offset = CreateGEP(usym_t, buf, { getInt64(0), getInt32(0) });
   Value *pid_offset = CreateGEP(usym_t, buf, { getInt64(0), getInt32(1) });
+  Value *probeid_offset = CreateGEP(usym_t, buf, { getInt64(0), getInt32(2) });
 
   CreateStore(val, addr_offset);
   CreateStore(pid, pid_offset);
+  CreateStore(probe_id_val, probeid_offset);
   return buf;
 }
 
@@ -964,12 +971,30 @@ Value *IRBuilderBPF::CreateStrcontains(Value *val1,
   return result;
 }
 
-CallInst *IRBuilderBPF::CreateGetNs(bool boot_time, const location &loc)
+CallInst *IRBuilderBPF::CreateGetNs(TimestampMode ts, const location &loc)
 {
-  // u64 ktime_get_ns()
+  libbpf::bpf_func_id fn;
+  switch (ts)
+  {
+    case TimestampMode::monotonic:
+      fn = libbpf::BPF_FUNC_ktime_get_ns;
+      break;
+    case TimestampMode::boot:
+      fn = bpftrace_.feature_->has_helper_ktime_get_boot_ns()
+               ? libbpf::BPF_FUNC_ktime_get_boot_ns
+               : libbpf::BPF_FUNC_ktime_get_ns;
+      break;
+    case TimestampMode::tai:
+      fn = libbpf::BPF_FUNC_ktime_get_tai_ns;
+      break;
+    case TimestampMode::sw_tai:
+      LOG(BUG) << "Invalid timestamp mode: "
+               << std::to_string(
+                      static_cast<std::underlying_type_t<TimestampMode>>(ts));
+  }
+
+  // u64 ktime_get_*ns()
   // Return: current ktime
-  auto fn = boot_time ? libbpf::BPF_FUNC_ktime_get_boot_ns
-                      : libbpf::BPF_FUNC_ktime_get_ns;
   FunctionType *gettime_func_type = FunctionType::get(getInt64Ty(), false);
   return CreateHelperCall(fn, gettime_func_type, {}, "get_ns", &loc);
 }
@@ -1315,6 +1340,18 @@ CallInst *IRBuilderBPF::CreateGetStackId(Value *ctx,
   return call;
 }
 
+CallInst *IRBuilderBPF::CreateGetFuncIp(const location &loc)
+{
+  // u64 bpf_get_func_ip(void *ctx)
+  // Return: Address of the traced function.
+  FunctionType *getfuncip_func_type = FunctionType::get(getInt64Ty(), false);
+  return CreateHelperCall(libbpf::BPF_FUNC_get_func_ip,
+                          getfuncip_func_type,
+                          {},
+                          "get_func_ip",
+                          &loc);
+}
+
 void IRBuilderBPF::CreateGetCurrentComm(Value *ctx,
                                         AllocaInst *buf,
                                         size_t size,
@@ -1336,14 +1373,107 @@ void IRBuilderBPF::CreateGetCurrentComm(Value *ctx,
   CreateHelperErrorCond(ctx, call, libbpf::BPF_FUNC_get_current_comm, loc);
 }
 
+void IRBuilderBPF::CreateOutput(Value *ctx,
+                                Value *data,
+                                size_t size,
+                                const location *loc)
+{
+  assert(ctx && ctx->getType() == getInt8PtrTy());
+  assert(data && data->getType()->isPointerTy());
+
+  if (bpftrace_.feature_->has_map_ringbuf())
+  {
+    CreateRingbufOutput(data, size, loc);
+  }
+  else
+  {
+    CreatePerfEventOutput(ctx, data, size, loc);
+  }
+}
+
+void IRBuilderBPF::CreateRingbufOutput(Value *data,
+                                       size_t size,
+                                       const location *loc)
+{
+  Value *map_ptr = CreateBpfPseudoCallId(
+      bpftrace_.maps[MapManager::Type::Ringbuf].value()->id);
+
+  // long bpf_ringbuf_output(void *ringbuf, void *data, u64 size, u64 flags)
+  FunctionType *ringbuf_output_func_type = FunctionType::get(
+      getInt64Ty(),
+      { map_ptr->getType(), data->getType(), getInt64Ty(), getInt64Ty() },
+      false);
+
+  Value *ret = CreateHelperCall(libbpf::BPF_FUNC_ringbuf_output,
+                                ringbuf_output_func_type,
+                                { map_ptr, data, getInt64(size), getInt64(0) },
+                                "ringbuf_output",
+                                loc);
+
+  Function *parent = GetInsertBlock()->getParent();
+  BasicBlock *loss_block = BasicBlock::Create(module_.getContext(),
+                                              "event_loss_counter",
+                                              parent);
+  BasicBlock *merge_block = BasicBlock::Create(module_.getContext(),
+                                               "counter_merge",
+                                               parent);
+  Value *condition = CreateICmpSLT(ret, getInt64(0), "ringbuf_loss");
+  CreateCondBr(condition, loss_block, merge_block);
+
+  SetInsertPoint(loss_block);
+  CreateAtomicIncCounter(
+      bpftrace_.maps[MapManager::Type::RingbufLossCounter].value()->id,
+      bpftrace_.rb_loss_cnt_key_);
+  CreateBr(merge_block);
+
+  SetInsertPoint(merge_block);
+}
+
+void IRBuilderBPF::CreateAtomicIncCounter(int mapid, uint32_t idx)
+{
+  AllocaInst *key = CreateAllocaBPF(getInt32Ty(), "key");
+  CreateStore(getInt32(idx), key);
+
+  CallInst *call = createMapLookup(mapid, key);
+  Function *parent = GetInsertBlock()->getParent();
+  BasicBlock *lookup_success_block = BasicBlock::Create(module_.getContext(),
+                                                        "lookup_success",
+                                                        parent);
+  BasicBlock *lookup_failure_block = BasicBlock::Create(module_.getContext(),
+                                                        "lookup_failure",
+                                                        parent);
+  BasicBlock *lookup_merge_block = BasicBlock::Create(module_.getContext(),
+                                                      "lookup_merge",
+                                                      parent);
+
+  Value *condition = CreateICmpNE(
+      CreateIntCast(call, getInt8PtrTy(), true),
+      ConstantExpr::getCast(Instruction::IntToPtr, getInt64(0), getInt8PtrTy()),
+      "map_lookup_cond");
+  CreateCondBr(condition, lookup_success_block, lookup_failure_block);
+
+  SetInsertPoint(lookup_success_block);
+  Value *val = CreatePointerCast(call, getInt64Ty()->getPointerTo());
+  CREATE_ATOMIC_RMW(AtomicRMWInst::BinOp::Add,
+                    val,
+                    getInt64(1),
+                    1,
+                    AtomicOrdering::SequentiallyConsistent);
+  CreateBr(lookup_merge_block);
+
+  SetInsertPoint(lookup_failure_block);
+  // ignore lookup failure
+  CreateBr(lookup_merge_block);
+
+  SetInsertPoint(lookup_merge_block);
+  CreateLifetimeEnd(key);
+}
+
 void IRBuilderBPF::CreatePerfEventOutput(Value *ctx,
                                          Value *data,
                                          size_t size,
                                          const location *loc)
 {
-  assert(ctx && ctx->getType() == getInt8PtrTy());
-  assert(data && data->getType()->isPointerTy());
-
   Value *map_ptr = CreateBpfPseudoCallId(
       bpftrace_.maps[MapManager::Type::PerfEvent].value()->id);
 
@@ -1489,6 +1619,40 @@ Value *IRBuilderBPF::CreateRawTracepointArg(Value *ctx,
   return expr;
 }
 
+Value *IRBuilderBPF::CreateUprobeArgsRecord(Value *ctx,
+                                            const SizedType &args_type)
+{
+  assert(args_type.IsRecordTy());
+
+  auto *args_t = UprobeArgsType(args_type);
+  AllocaInst *result = CreateAllocaBPF(args_t, "args");
+
+  for (auto &arg : args_type.GetFields())
+  {
+    assert(arg.type.is_funcarg);
+    Value *arg_read = CreateRegisterRead(
+        ctx, "arg" + std::to_string(arg.type.funcarg_idx));
+    if (arg.type.GetSize() != 8)
+      arg_read = CreateTrunc(arg_read, GetType(arg.type));
+    CreateStore(arg_read,
+                CreateGEP(args_t,
+                          result,
+                          { getInt64(0), getInt32(arg.type.funcarg_idx) }));
+  }
+  return result;
+}
+
+llvm::Type *IRBuilderBPF::UprobeArgsType(const SizedType &args_type)
+{
+  auto type_name = args_type.GetName();
+  type_name.erase(0, strlen("struct "));
+
+  std::vector<llvm::Type *> arg_types;
+  for (auto &arg : args_type.GetFields())
+    arg_types.push_back(GetType(arg.type));
+  return GetStructType(type_name, arg_types, false);
+}
+
 Value *IRBuilderBPF::CreateRegisterRead(Value *ctx, const std::string &builtin)
 {
   int offset;
@@ -1584,7 +1748,7 @@ void IRBuilderBPF::CreateHelperError(Value *ctx,
 
   auto &layout = module_.getDataLayout();
   auto struct_size = layout.getTypeAllocSize(helper_error_struct);
-  CreatePerfEventOutput(ctx, buf, struct_size, &loc);
+  CreateOutput(ctx, buf, struct_size, &loc);
   CreateLifetimeEnd(buf);
 }
 
@@ -1773,7 +1937,7 @@ llvm::Type *IRBuilderBPF::getPointerStorageTy(AddrSpace as)
 
 llvm::Type *IRBuilderBPF::getKernelPointerStorageTy()
 {
-  static int ptr_width = get_kernel_ptr_width();
+  static int ptr_width = arch::get_kernel_ptr_width();
 
   return getIntNTy(ptr_width);
 }
