@@ -38,6 +38,14 @@
   CreateMemSet((ptr), (val), (size), (align))
 #endif
 
+#if LLVM_VERSION_MAJOR >= 13
+#define CREATE_ATOMIC_RMW(op, ptr, val, align, order)                          \
+  CreateAtomicRMW((op), (ptr), (val), MaybeAlign((align)), (order))
+#else
+#define CREATE_ATOMIC_RMW(op, ptr, val, align, order)                          \
+  CreateAtomicRMW((op), (ptr), (val), (order))
+#endif
+
 namespace bpftrace {
 namespace ast {
 
@@ -151,7 +159,7 @@ public:
                                const bool inverse,
                                const location &loc,
                                MDNode *metadata);
-  CallInst *CreateGetNs(bool boot_time, const location &loc);
+  CallInst *CreateGetNs(TimestampMode ts, const location &loc);
   CallInst *CreateGetPidTgid(const location &loc);
   CallInst *CreateGetCurrentCgroupId(const location &loc);
   CallInst *CreateGetUidGid(const location &loc);
@@ -160,6 +168,7 @@ public:
   CallInst *CreateGetCurrentTask(const location &loc);
   CallInst *CreateGetRandom(const location &loc);
   CallInst   *CreateGetStackId(Value *ctx, bool ustack, StackType stack_type, const location& loc);
+  CallInst *CreateGetFuncIp(const location &loc);
   CallInst   *CreateGetJoinMap(Value *ctx, const location& loc);
   CallInst *CreateHelperCall(libbpf::bpf_func_id func_id,
                              FunctionType *helper_type,
@@ -171,10 +180,11 @@ public:
                        ArrayRef<Value *> args,
                        const Twine &Name);
   void        CreateGetCurrentComm(Value *ctx, AllocaInst *buf, size_t size, const location& loc);
-  void CreatePerfEventOutput(Value *ctx,
-                             Value *data,
-                             size_t size,
-                             const location *loc = nullptr);
+  void CreateOutput(Value *ctx,
+                    Value *data,
+                    size_t size,
+                    const location *loc = nullptr);
+  void CreateAtomicIncCounter(int mapfd, uint32_t idx);
   void CreateTracePrintk(Value *fmt,
                          Value *fmt_size,
                          const std::vector<Value *> &values,
@@ -184,11 +194,13 @@ public:
   void        CreateHelperError(Value *ctx, Value *return_value, libbpf::bpf_func_id func_id, const location& loc);
   void        CreateHelperErrorCond(Value *ctx, Value *return_value, libbpf::bpf_func_id func_id, const location& loc, bool compare_zero=false);
   StructType *GetStructType(std::string name, const std::vector<llvm::Type *> & elements, bool packed = false);
-  AllocaInst *CreateUSym(llvm::Value *val, const location &loc);
+  AllocaInst *CreateUSym(llvm::Value *val, int probe_id, const location &loc);
   Value *CreateRegisterRead(Value *ctx, const std::string &builtin);
   Value *CreateRegisterRead(Value *ctx, int offset, const std::string &name);
   Value      *CreatKFuncArg(Value *ctx, SizedType& type, std::string& name);
   Value *CreateRawTracepointArg(Value *ctx, const std::string &builtin);
+  Value *CreateUprobeArgsRecord(Value *ctx, const SizedType &args_type);
+  llvm::Type *UprobeArgsType(const SizedType &args_type);
   CallInst *CreateSkbOutput(Value *skb,
                             Value *len,
                             AllocaInst *data,
@@ -239,6 +251,13 @@ private:
 
   llvm::Type *getKernelPointerStorageTy();
   llvm::Type *getUserPointerStorageTy();
+  void CreateRingbufOutput(Value *data,
+                           size_t size,
+                           const location *loc = nullptr);
+  void CreatePerfEventOutput(Value *ctx,
+                             Value *data,
+                             size_t size,
+                             const location *loc = nullptr);
 
   std::map<std::string, StructType *> structs_;
 };
