@@ -65,6 +65,13 @@ void ResourceAnalyser::visit(Builtin &builtin)
   {
     resources_.stackid_maps.insert(StackType{});
   }
+
+  if (uses_usym_table(builtin.ident))
+  {
+    // mark probe as using usym, so that the symbol table can be pre-loaded
+    // and symbols resolved even when unavailable at resolution time
+    resources_.probes_using_usym.insert(probe_);
+  }
 }
 
 void ResourceAnalyser::visit(Call &call)
@@ -192,6 +199,14 @@ void ResourceAnalyser::visit(Call &call)
     Integer &offset = static_cast<Integer &>(offset_arg);
 
     resources_.skboutput_args_.emplace_back(file.str, offset.n);
+    resources_.needs_perf_event_map = true;
+  }
+
+  if (uses_usym_table(call.func))
+  {
+    // mark probe as using usym, so that the symbol table can be pre-loaded
+    // and symbols resolved even when unavailable at resolution time
+    resources_.probes_using_usym.insert(probe_);
   }
 }
 
@@ -216,6 +231,11 @@ void ResourceAnalyser::prepare_mapped_printf_ids()
   }
 }
 
+bool ResourceAnalyser::uses_usym_table(const std::string &fun)
+{
+  return fun == "usym" || fun == "func" || fun == "ustack";
+}
+
 Pass CreateResourcePass()
 {
   auto fn = [](Node &n, PassContext &ctx) {
