@@ -43,7 +43,7 @@ private:
   std::string eventname() const;
   static std::string sanitise(const std::string &str);
   void resolve_offset_kprobe(bool safe_mode);
-  void resolve_offset_uprobe(bool safe_mode);
+  bool resolve_offset_uprobe(bool safe_mode);
   void load_prog(BPFfeature &feature);
   void attach_multi_kprobe(void);
   void attach_kprobe(bool safe_mode);
@@ -73,6 +73,8 @@ private:
   int detach_kfunc(void);
   void attach_iter(void);
   int detach_iter(void);
+  void attach_raw_tracepoint(void);
+  int detach_raw_tracepoint(void);
 
   static std::map<std::string, int> cached_prog_fds_;
   bool use_cached_progfd(void);
