@@ -177,6 +177,14 @@ bool BPFfeature::detect_map(enum libbpf::bpf_map_type map_type)
     case libbpf::BPF_MAP_TYPE_STACK_TRACE:
       value_size = 8;
       break;
+    case libbpf::BPF_MAP_TYPE_RINGBUF:
+      // values from libbpf/src/libbpf_probes.c
+      // default pagesize 4KB
+      // default perf_rb_pages 64
+      key_size = 0;
+      value_size = 0;
+      max_entries = sysconf(_SC_PAGE_SIZE);
+      break;
     default:
       break;
   }
@@ -492,6 +500,8 @@ std::string BPFfeature::report(void)
       << "  get_boot_ns: " << to_str(has_helper_ktime_get_boot_ns())
       << "  dpath: " << to_str(has_d_path())
       << "  skboutput: " << to_str(has_skb_output())
+      << "  get_tai_ns: " << to_str(has_helper_ktime_get_tai_ns())
+      << "  get_func_ip: " << to_str(has_helper_get_func_ip())
 
       << std::endl;
 
@@ -511,18 +521,16 @@ std::string BPFfeature::report(void)
       << "  percpu array: " << to_str(has_map_percpu_array())
       << "  stack_trace: " << to_str(has_map_stack_trace())
       << "  perf_event_array: " << to_str(has_map_perf_event_array())
-      << std::endl;
+      << "  ringbuf: " << to_str(has_map_ringbuf()) << std::endl;
 
   buf << "Probe types" << std::endl
       << "  kprobe: " << to_str(has_prog_kprobe())
       << "  tracepoint: " << to_str(has_prog_tracepoint())
       << "  perf_event: " << to_str(has_prog_perf_event())
       << "  kfunc: " << to_str(has_kfunc())
-      << "  iter:task: " << to_str(has_prog_iter_task())
-      << "  iter:task_file: " << to_str(has_prog_iter_task_file())
-      << "  iter:task_vma: " << to_str(has_prog_iter_task_vma())
       << "  kprobe_multi: " << to_str(has_kprobe_multi())
-      << "  raw_tp_special: " << to_str(has_raw_tp_special()) << std::endl;
+      << "  raw_tp_special: " << to_str(has_raw_tp_special())
+      << "  iter: " << to_str(has_iter("task")) << std::endl;
 
   return buf.str();
 }
@@ -588,4 +596,12 @@ not_support:
   return *has_module_btf_;
 }
 
+bool BPFfeature::has_iter(std::string name)
+{
+  auto tracing_name = "bpf_iter_" + name;
+  return detect_prog_type(libbpf::BPF_PROG_TYPE_TRACING,
+                          tracing_name.c_str(),
+                          libbpf::BPF_TRACE_ITER);
+}
+
 } // namespace bpftrace
