@@ -54,9 +54,6 @@ public:                                                                        \
 #define DEFINE_PROG_TEST(var, progtype)                                        \
   __DEFINE_PROG_TEST(var, progtype, NULL, std::nullopt)
 
-#define DEFINE_PROG_TEST_FUNC(var, progtype, name, attach_type)                \
-  __DEFINE_PROG_TEST(var, progtype, name, attach_type)
-
 class BPFfeature
 {
 public:
@@ -85,6 +82,7 @@ public:
   bool has_raw_tp_special();
   bool has_prog_kfunc();
   bool has_module_btf();
+  bool has_iter(std::string name);
 
   std::string report(void);
 
@@ -94,6 +92,7 @@ public:
   DEFINE_MAP_TEST(percpu_hash, libbpf::BPF_MAP_TYPE_ARRAY);
   DEFINE_MAP_TEST(stack_trace, libbpf::BPF_MAP_TYPE_STACK_TRACE);
   DEFINE_MAP_TEST(perf_event_array, libbpf::BPF_MAP_TYPE_PERF_EVENT_ARRAY);
+  DEFINE_MAP_TEST(ringbuf, libbpf::BPF_MAP_TYPE_RINGBUF);
   DEFINE_HELPER_TEST(send_signal, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_HELPER_TEST(override_return, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_HELPER_TEST(get_current_cgroup_id, libbpf::BPF_PROG_TYPE_KPROBE);
@@ -104,21 +103,11 @@ public:
   DEFINE_HELPER_TEST(probe_read_user_str, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_HELPER_TEST(probe_read_kernel_str, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_HELPER_TEST(ktime_get_boot_ns, libbpf::BPF_PROG_TYPE_KPROBE);
+  DEFINE_HELPER_TEST(ktime_get_tai_ns, libbpf::BPF_PROG_TYPE_KPROBE);
+  DEFINE_HELPER_TEST(get_func_ip, libbpf::BPF_PROG_TYPE_TRACING);
   DEFINE_PROG_TEST(kprobe, libbpf::BPF_PROG_TYPE_KPROBE);
   DEFINE_PROG_TEST(tracepoint, libbpf::BPF_PROG_TYPE_TRACEPOINT);
   DEFINE_PROG_TEST(perf_event, libbpf::BPF_PROG_TYPE_PERF_EVENT);
-  DEFINE_PROG_TEST_FUNC(iter_task,
-                        libbpf::BPF_PROG_TYPE_TRACING,
-                        "bpf_iter_task",
-                        libbpf::BPF_TRACE_ITER);
-  DEFINE_PROG_TEST_FUNC(iter_task_file,
-                        libbpf::BPF_PROG_TYPE_TRACING,
-                        "bpf_iter_task_file",
-                        libbpf::BPF_TRACE_ITER);
-  DEFINE_PROG_TEST_FUNC(iter_task_vma,
-                        libbpf::BPF_PROG_TYPE_TRACING,
-                        "bpf_iter_task_vma",
-                        libbpf::BPF_TRACE_ITER);
 
 protected:
   std::optional<bool> has_loop_;
