@@ -1,5 +1,6 @@
 #include <arpa/inet.h>
 #include <cassert>
+#include <cerrno>
 #include <cstdio>
 #include <cstring>
 #include <ctime>
@@ -25,6 +26,7 @@
 #include <bcc/bcc_syms.h>
 #include <bcc/perf_reader.h>
 #include <bpf/bpf.h>
+#include <bpf/libbpf.h>
 
 #include "ast/async_event_types.h"
 #include "bpftrace.h"
@@ -49,6 +51,12 @@ bool bt_verbose = false;
 bool bt_verbose2 = false;
 volatile sig_atomic_t BPFtrace::exitsig_recv = false;
 volatile sig_atomic_t BPFtrace::sigusr1_recv = false;
+struct bcc_symbol_option symopts = {
+  .use_debug_file = 1,
+  .check_debug_file_crc = 1,
+  .lazy_symbolize = 0,
+  .use_symbol_type = BCC_SYM_ALL_TYPES,
+};
 
 BPFtrace::~BPFtrace()
 {
@@ -58,6 +66,12 @@ BPFtrace::~BPFtrace()
       bcc_free_symcache(pair.second.second, pair.second.first);
   }
 
+  for (const auto &pair : pid_sym_)
+  {
+    if (pair.second)
+      bcc_free_symcache(pair.second, pair.first);
+  }
+
   if (ksyms_)
     bcc_free_symcache(ksyms_, -1);
 }
@@ -164,18 +178,19 @@ int BPFtrace::add_probe(ast::Probe &p)
 
       struct symbol sym = {};
       int err = resolve_uname(attach_point->func, &sym, attach_point->target);
-      if (err < 0 || sym.address == 0)
+
+      if (attach_point->lang == "cpp")
       {
         // As the C++ language supports function overload, a given function name
         // (without parameters) could have multiple matches even when no
         // wildcards are used.
         matches = probe_matcher_->get_matches_for_ap(*attach_point);
-        attach_funcs.insert(attach_funcs.end(), matches.begin(), matches.end());
-      }
-      else
-      {
-        attach_funcs.push_back(attach_point->target + ":" + attach_point->func);
       }
+
+      if (err >= 0 && sym.address != 0)
+        matches.insert(attach_point->target + ":" + attach_point->func);
+
+      attach_funcs.insert(attach_funcs.end(), matches.begin(), matches.end());
     }
     else
     {
@@ -187,8 +202,7 @@ int BPFtrace::add_probe(ast::Probe &p)
                probetype(attach_point->provider) == ProbeType::uprobe ||
                probetype(attach_point->provider) == ProbeType::uretprobe ||
                probetype(attach_point->provider) == ProbeType::kfunc ||
-               probetype(attach_point->provider) == ProbeType::kretfunc ||
-               probetype(attach_point->provider) == ProbeType::rawtracepoint)
+               probetype(attach_point->provider) == ProbeType::kretfunc)
         attach_funcs.push_back(attach_point->target + ":" + attach_point->func);
       else
         attach_funcs.push_back(attach_point->func);
@@ -235,8 +249,7 @@ int BPFtrace::add_probe(ast::Probe &p)
                probetype(attach_point->provider) == ProbeType::uprobe ||
                probetype(attach_point->provider) == ProbeType::uretprobe ||
                probetype(attach_point->provider) == ProbeType::kfunc ||
-               probetype(attach_point->provider) == ProbeType::kretfunc ||
-               probetype(attach_point->provider) == ProbeType::rawtracepoint)
+               probetype(attach_point->provider) == ProbeType::kretfunc)
       {
         // tracepoint, uprobe, and k(ret)func probes specify both a target and
         // a function name.
@@ -305,6 +318,29 @@ int BPFtrace::add_probe(ast::Probe &p)
         resources.probes.push_back(probe);
       }
     }
+
+    if (resources.probes_using_usym.find(&p) !=
+            resources.probes_using_usym.end() &&
+        bcc_elf_is_exe(attach_point->target.c_str()))
+    {
+      // preload symbol table for executable to make it available even if the
+      // binary is not present at symbol resolution time
+      // note: this only makes sense with ASLR disabled, since with ASLR offsets
+      // might be different
+      if (user_symbol_cache_type_ == UserSymbolCacheType::per_program &&
+          symbol_table_cache_.find(attach_point->target) ==
+              symbol_table_cache_.end())
+        symbol_table_cache_[attach_point->target] = get_symbol_table_for_elf(
+            attach_point->target);
+
+      if (user_symbol_cache_type_ == UserSymbolCacheType::per_pid)
+        // preload symbol tables from running processes
+        // this allows symbol resolution for processes that are running at probe
+        // attach time, but not at symbol resolution time, even with ASLR
+        // enabled, since BCC symcache records the offsets
+        for (int pid : get_pids_for_program(attach_point->target))
+          pid_sym_[pid] = bcc_symcache_new(pid, &symopts);
+    }
   }
 
   return 0;
@@ -603,6 +639,12 @@ void perf_event_printer(void *cb_cookie, void *data, int size)
                           false);
 }
 
+int ringbuf_printer(void *cb_cookie, void *data, size_t size)
+{
+  perf_event_printer(cb_cookie, data, size);
+  return 0;
+}
+
 std::vector<std::unique_ptr<IPrintable>> BPFtrace::get_arg_values(const std::vector<Field> &args, uint8_t* arg_data)
 {
   std::vector<std::unique_ptr<IPrintable>> arg_values;
@@ -688,11 +730,10 @@ std::vector<std::unique_ptr<IPrintable>> BPFtrace::get_arg_values(const std::vec
             resolve_ksym(*reinterpret_cast<uint64_t*>(arg_data+arg.offset))));
         break;
       case Type::usym:
-        arg_values.push_back(
-          std::make_unique<PrintableString>(
-            resolve_usym(
-              *reinterpret_cast<uint64_t*>(arg_data+arg.offset),
-              *reinterpret_cast<uint64_t*>(arg_data+arg.offset + 8))));
+        arg_values.push_back(std::make_unique<PrintableString>(resolve_usym(
+            *reinterpret_cast<uint64_t *>(arg_data + arg.offset),
+            *reinterpret_cast<uint64_t *>(arg_data + arg.offset + 8),
+            *reinterpret_cast<uint64_t *>(arg_data + arg.offset + 16))));
         break;
       case Type::inet:
         arg_values.push_back(
@@ -714,28 +755,32 @@ std::vector<std::unique_ptr<IPrintable>> BPFtrace::get_arg_values(const std::vec
               *reinterpret_cast<uint64_t*>(arg_data+arg.offset))));
         break;
       case Type::kstack:
-        arg_values.push_back(
-          std::make_unique<PrintableString>(
-            get_stack(
-              *reinterpret_cast<uint64_t*>(arg_data+arg.offset),
-              false,
-              arg.type.stack_type, 8)));
+        arg_values.push_back(std::make_unique<PrintableString>(
+            get_stack(*reinterpret_cast<int64_t *>(arg_data + arg.offset),
+                      -1,
+                      -1,
+                      false,
+                      arg.type.stack_type,
+                      8)));
         break;
       case Type::ustack:
-        arg_values.push_back(
-          std::make_unique<PrintableString>(
-            get_stack(
-              *reinterpret_cast<uint64_t*>(arg_data+arg.offset),
-              true,
-              arg.type.stack_type, 8)));
+        arg_values.push_back(std::make_unique<PrintableString>(
+            get_stack(*reinterpret_cast<int64_t *>(arg_data + arg.offset),
+                      *reinterpret_cast<int32_t *>(arg_data + arg.offset + 8),
+                      *reinterpret_cast<int32_t *>(arg_data + arg.offset + 12),
+                      true,
+                      arg.type.stack_type,
+                      8)));
         break;
       case Type::timestamp:
         arg_values.push_back(
             std::make_unique<PrintableString>(resolve_timestamp(
+                reinterpret_cast<AsyncEvent::Strftime *>(arg_data + arg.offset)
+                    ->mode,
                 reinterpret_cast<AsyncEvent::Strftime *>(arg_data + arg.offset)
                     ->strftime_id,
                 reinterpret_cast<AsyncEvent::Strftime *>(arg_data + arg.offset)
-                    ->nsecs_since_boot)));
+                    ->nsecs)));
         break;
       case Type::pointer:
         arg_values.push_back(std::make_unique<PrintableInt>(
@@ -998,7 +1043,6 @@ bool attach_reverse(const Probe &p)
     case ProbeType::software:
     case ProbeType::kfunc:
     case ProbeType::iter:
-    case ProbeType::rawtracepoint:
       return true;
     case ProbeType::kretfunc:
     case ProbeType::kretprobe:
@@ -1008,6 +1052,7 @@ bool attach_reverse(const Probe &p)
     case ProbeType::watchpoint:
     case ProbeType::asyncwatchpoint:
     case ProbeType::hardware:
+    case ProbeType::rawtracepoint:
       return false;
     case ProbeType::invalid:
       LOG(FATAL) << "Unknown probe type";
@@ -1018,7 +1063,7 @@ bool attach_reverse(const Probe &p)
 
 int BPFtrace::run_special_probe(std::string name,
                                 BpfBytecode &bytecode,
-                                void (*trigger)(void))
+                                trigger_fn_t trigger)
 {
   for (auto probe = resources.special_probes.rbegin();
        probe != resources.special_probes.rend();
@@ -1151,7 +1196,7 @@ int BPFtrace::run(BpfBytecode bytecode)
 
   bytecode_ = std::move(bytecode);
 
-  err = setup_perf_events();
+  err = setup_output();
   if (err)
     return err;
 
@@ -1172,7 +1217,12 @@ int BPFtrace::run(BpfBytecode bytecode)
     }
   }
 
-  if (run_special_probe("BEGIN_trigger", bytecode_, BEGIN_trigger))
+  // XXX: cast is a workaround for a gcc bug that causes an "invalid conversion"
+  // error here in case the trigger function has the `target` attribute set (see
+  // triggers.h): https://gcc.godbolt.org/z/44b5MjdbT
+  if (run_special_probe("BEGIN_trigger",
+                        bytecode_,
+                        reinterpret_cast<trigger_fn_t>(BEGIN_trigger)))
     return -1;
 
   if (child_ && has_usdt_)
@@ -1252,7 +1302,7 @@ int BPFtrace::run(BpfBytecode bytecode)
   }
   else
   {
-    poll_perf_events();
+    poll_output();
   }
 
   attached_probes_.clear();
@@ -1261,17 +1311,33 @@ int BPFtrace::run(BpfBytecode bytecode)
   finalize_ = false;
   exitsig_recv = false;
 
-  if (run_special_probe("END_trigger", bytecode_, END_trigger))
+  if (run_special_probe("END_trigger",
+                        bytecode_,
+                        reinterpret_cast<trigger_fn_t>(END_trigger)))
     return -1;
 
-  poll_perf_events(/* drain */ true);
+  poll_output(/* drain */ true);
 
-  // Calls perf_reader_free() on all open perf buffers.
-  open_perf_buffers_.clear();
+  teardown_output();
 
   return 0;
 }
 
+int BPFtrace::setup_output()
+{
+  if (is_ringbuf_enabled())
+  {
+    int err = setup_ringbuf();
+    if (err)
+      return err;
+  }
+  if (is_perf_event_enabled())
+  {
+    return setup_perf_events();
+  }
+  return 0;
+}
+
 int BPFtrace::setup_perf_events()
 {
   epollfd_ = epoll_create1(EPOLL_CLOEXEC);
@@ -1313,39 +1379,98 @@ int BPFtrace::setup_perf_events()
   return 0;
 }
 
-void BPFtrace::poll_perf_events(bool drain)
+int BPFtrace::setup_ringbuf()
 {
-  const int timeout_ms = 100;
+  ringbuf_ = static_cast<struct ring_buffer *>(
+      ring_buffer__new(maps[MapManager::Type::Ringbuf].value()->mapfd_,
+                       ringbuf_printer,
+                       this,
+                       nullptr));
+  if (bpf_update_elem(
+          maps[MapManager::Type::RingbufLossCounter].value()->mapfd_,
+          const_cast<uint32_t *>(&rb_loss_cnt_key_),
+          const_cast<uint64_t *>(&rb_loss_cnt_val_),
+          0))
+  {
+    LOG(ERROR) << "fail to init ringbuf loss counter";
+    return -1;
+  }
+  return 0;
+}
 
-  if (epollfd_ < 0)
+void BPFtrace::teardown_output()
+{
+  if (is_ringbuf_enabled())
+    ring_buffer__free(ringbuf_);
+
+  if (is_perf_event_enabled())
+    // Calls perf_reader_free() on all open perf buffers.
+    open_perf_buffers_.clear();
+}
+
+void BPFtrace::poll_output(bool drain)
+{
+  int ready;
+  bool do_poll_perf_event = is_perf_event_enabled();
+  bool do_poll_ringbuf = is_ringbuf_enabled();
+  auto should_retry = [](int ready) {
+    // epoll_wait will set errno to EINTR if an interrupt received, it is
+    // retryable if not caused by SIGINT. ring_buffer__poll does not set errno,
+    // we will keep retrying till SIGINT.
+    return ready < 0 && (errno == 0 || errno == EINTR) &&
+           !BPFtrace::exitsig_recv;
+  };
+  auto should_stop = [this, drain](int ready) {
+    // Stop if either
+    //   * an exit signal is received
+    //   * epoll_wait has encountered an error (eg signal delivery)
+    //   * there's no events left and we've been instructed to drain or
+    //     finalization has been requested through exit() builtin.
+    return BPFtrace::exitsig_recv || ready < 0 ||
+           (ready == 0 && (drain || finalize_));
+  };
+
+  if (do_poll_perf_event && epollfd_ < 0)
   {
     LOG(ERROR) << "Invalid epollfd " << epollfd_;
     return;
   }
 
-  auto events = std::vector<struct epoll_event>(online_cpus_);
   while (true)
   {
-    int ready = epoll_wait(epollfd_, events.data(), online_cpus_, timeout_ms);
-    if (ready < 0 && errno == EINTR && !BPFtrace::exitsig_recv) {
-      // We received an interrupt not caused by SIGINT, skip and run again
-      continue;
+    if (do_poll_perf_event)
+    {
+      ready = poll_perf_events();
+      if (should_retry(ready))
+      {
+        if (!do_poll_ringbuf)
+          continue;
+      }
+      if (should_stop(ready))
+      {
+        do_poll_perf_event = false;
+      }
     }
 
-    // Return if either
-    //   * epoll_wait has encountered an error (eg signal delivery)
-    //   * There's no events left and we've been instructed to drain or
-    //     finalization has been requested through exit() builtin.
-    if (ready < 0 || (ready == 0 && (drain || finalize_)))
+    if (do_poll_ringbuf)
+    {
+      // print loss events
+      handle_ringbuf_loss();
+      ready = ring_buffer__poll(ringbuf_, timeout_ms);
+      if (should_retry(ready))
+      {
+        continue;
+      }
+      if (should_stop(ready))
+      {
+        do_poll_ringbuf = false;
+      }
+    }
+    if (!do_poll_perf_event && !do_poll_ringbuf)
     {
       return;
     }
 
-    for (int i=0; i<ready; i++)
-    {
-      perf_reader_event_read((perf_reader*)events[i].data.ptr);
-    }
-
     // If we are tracing a specific pid and it has exited, we should exit
     // as well b/c otherwise we'd be tracing nothing.
     if ((procmon_ && !procmon_->is_alive()) || (child_ && !child_->is_alive()))
@@ -1363,6 +1488,46 @@ void BPFtrace::poll_perf_events(bool drain)
   return;
 }
 
+int BPFtrace::poll_perf_events()
+{
+  auto events = std::vector<struct epoll_event>(online_cpus_);
+  int ready = epoll_wait(epollfd_, events.data(), online_cpus_, timeout_ms);
+  if (ready <= 0)
+  {
+    return ready;
+  }
+  for (int i = 0; i < ready; i++)
+  {
+    perf_reader_event_read((perf_reader *)events[i].data.ptr);
+  }
+  return ready;
+}
+
+void BPFtrace::handle_ringbuf_loss()
+{
+  uint64_t current_value = 0;
+  if (bpf_lookup_elem(
+          maps[MapManager::Type::RingbufLossCounter].value()->mapfd_,
+          const_cast<uint32_t *>(&rb_loss_cnt_key_),
+          &current_value))
+  {
+    LOG(ERROR) << "fail to get ringbuf loss counter";
+  }
+  if (current_value)
+  {
+    if (current_value > ringbuf_loss_count_)
+    {
+      out_->lost_events(current_value - ringbuf_loss_count_);
+      ringbuf_loss_count_ = current_value;
+    }
+    else if (current_value < ringbuf_loss_count_)
+    {
+      LOG(ERROR) << "Invalid ringbuf loss count value: " << current_value
+                 << ", last seen: " << ringbuf_loss_count_;
+    }
+  }
+}
+
 int BPFtrace::print_maps()
 {
   for (auto &mapmap : maps)
@@ -1413,7 +1578,7 @@ int BPFtrace::clear_map(IMap &map)
   for (auto &key : keys)
   {
     int err = bpf_delete_elem(map.mapfd_, key.data());
-    if (err)
+    if (err && err != -ENOENT)
     {
       LOG(ERROR) << "failed to look up elem: " << err;
       return -1;
@@ -1459,7 +1624,7 @@ int BPFtrace::zero_map(IMap &map)
   {
     int err = bpf_update_elem(map.mapfd_, key.data(), zero.data(), BPF_EXIST);
 
-    if (err)
+    if (err && err != -ENOENT)
     {
       LOG(ERROR) << "failed to look up elem: " << err;
       return -1;
@@ -1498,7 +1663,7 @@ int BPFtrace::print_map(IMap &map, uint32_t top, uint32_t div)
     value_size *= nvalues;
     auto value = std::vector<uint8_t>(value_size);
     int err = bpf_lookup_elem(map.mapfd_, key.data(), value.data());
-    if (err == -1)
+    if (err == -ENOENT)
     {
       // key was removed by the eBPF program during bpf_get_next_key() and bpf_lookup_elem(),
       // let's skip this key
@@ -1584,7 +1749,7 @@ int BPFtrace::print_map_hist(IMap &map, uint32_t top, uint32_t div)
     int value_size = map.type_.GetSize() * nvalues;
     auto value = std::vector<uint8_t>(value_size);
     int err = bpf_lookup_elem(map.mapfd_, key.data(), value.data());
-    if (err == -1)
+    if (err == -ENOENT)
     {
       // key was removed by the eBPF program during bpf_get_next_key() and bpf_lookup_elem(),
       // let's skip this key
@@ -1663,7 +1828,7 @@ int BPFtrace::print_map_stats(IMap &map, uint32_t top, uint32_t div)
     int value_size = map.type_.GetSize() * nvalues;
     auto value = std::vector<uint8_t>(value_size);
     int err = bpf_lookup_elem(map.mapfd_, key.data(), value.data());
-    if (err == -1)
+    if (err == -ENOENT)
     {
       // key was removed by the eBPF program during bpf_get_next_key() and bpf_lookup_elem(),
       // let's skip this key
@@ -1752,10 +1917,13 @@ std::vector<uint8_t> BPFtrace::find_empty_key(IMap &map, size_t size) const
   throw std::runtime_error("Could not find empty key");
 }
 
-std::string BPFtrace::get_stack(uint64_t stackidpid, bool ustack, StackType stack_type, int indent)
+std::string BPFtrace::get_stack(int64_t stackid,
+                                int pid,
+                                int probe_id,
+                                bool ustack,
+                                StackType stack_type,
+                                int indent)
 {
-  int32_t stackid = stackidpid & 0xffffffff;
-  int pid = stackidpid >> 32;
   auto stack_trace = std::vector<uint64_t>(stack_type.limit);
   int err = bpf_lookup_elem(maps[stack_type].value()->mapfd_,
                             &stackid,
@@ -1786,7 +1954,8 @@ std::string BPFtrace::get_stack(uint64_t stackidpid, bool ustack, StackType stac
     if (!ustack)
       sym = resolve_ksym(addr, true);
     else
-      sym = resolve_usym(addr, pid, true, stack_type.mode == StackMode::perf);
+      sym = resolve_usym(
+          addr, pid, probe_id, true, stack_type.mode == StackMode::perf);
 
     switch (stack_type.mode) {
       case StackMode::bpftrace:
@@ -1825,7 +1994,7 @@ std::string BPFtrace::resolve_uid(uintptr_t addr) const
   {
     auto fields = split_string(line, ':');
 
-    if (fields[2] == uid)
+    if (fields.size() >= 3 && fields[2] == uid)
     {
       found = true;
       username = fields[0];
@@ -1837,21 +2006,32 @@ std::string BPFtrace::resolve_uid(uintptr_t addr) const
   return username;
 }
 
-std::string BPFtrace::resolve_timestamp(uint32_t strftime_id,
-                                        uint64_t nsecs_since_boot)
+std::string BPFtrace::resolve_timestamp(uint32_t mode,
+                                        uint32_t strftime_id,
+                                        uint64_t nsecs)
 {
   static const auto usec_regex = std::regex("%f");
+  TimestampMode ts_mode = static_cast<TimestampMode>(mode);
+  struct timespec zero = {};
+  struct timespec *basetime = &zero;
 
-  if (!boottime_)
+  if (ts_mode == TimestampMode::boot)
   {
-    LOG(ERROR) << "Cannot resolve timestamp due to failed boot time calcuation";
-    return "(?)";
+    if (!boottime_)
+    {
+      LOG(ERROR)
+          << "Cannot resolve timestamp due to failed boot time calcuation";
+      return "(?)";
+    }
+    else
+    {
+      basetime = &boottime_.value();
+    }
   }
 
   // Calculate and localize timestamp
   struct tm tmp;
-  time_t time = boottime_->tv_sec +
-                ((boottime_->tv_nsec + nsecs_since_boot) / 1e9);
+  time_t time = basetime->tv_sec + ((basetime->tv_nsec + nsecs) / 1e9);
   if (!localtime_r(&time, &tmp))
   {
     LOG(ERROR) << "localtime_r: " << strerror(errno);
@@ -1860,7 +2040,7 @@ std::string BPFtrace::resolve_timestamp(uint32_t strftime_id,
 
   // Process strftime() format string extensions
   const auto &raw_fmt = resources.strftime_args[strftime_id];
-  uint64_t us = ((boottime_->tv_nsec + nsecs_since_boot) % 1000000000) / 1000;
+  uint64_t us = ((basetime->tv_nsec + nsecs) % 1000000000) / 1000;
   char usecs_buf[7];
   snprintf(usecs_buf, sizeof(usecs_buf), "%06lu", us);
   auto fmt = std::regex_replace(raw_fmt, usec_regex, usecs_buf);
@@ -1958,7 +2138,8 @@ int BPFtrace::resolve_uname(const std::string &name,
   sym->name = name;
   struct bcc_symbol_option option;
   memset(&option, 0, sizeof(option));
-  option.use_symbol_type = (1 << STT_OBJECT);
+  option.use_symbol_type = (1 << STT_OBJECT | 1 << STT_FUNC |
+                            1 << STT_GNU_IFUNC);
 
   return bcc_elf_foreach_sym(path.c_str(), sym_resolve_callback, &option, sym);
 }
@@ -2113,23 +2294,64 @@ bool BPFtrace::is_aslr_enabled(int pid)
   return false;
 }
 
-std::string BPFtrace::resolve_usym(uintptr_t addr, int pid, bool show_offset, bool show_module)
+std::string BPFtrace::resolve_usym(uintptr_t addr,
+                                   int pid,
+                                   int probe_id,
+                                   bool show_offset,
+                                   bool show_module)
 {
   struct bcc_symbol usym;
   std::ostringstream symbol;
   void *psyms = nullptr;
-  struct bcc_symbol_option symopts;
-
-  memset(&symopts, 0, sizeof(symopts));
-  symopts.use_debug_file = 1;
-  symopts.check_debug_file_crc = 1;
-  symopts.use_symbol_type = BCC_SYM_ALL_TYPES;
 
   if (resolve_user_symbols_)
   {
-    if (cache_user_symbols_)
+    std::string pid_exe = get_pid_exe(pid);
+    if (pid_exe.empty() && probe_id != -1)
     {
-      std::string pid_exe = get_pid_exe(pid);
+      // sometimes program cannot be determined from PID, typically when the
+      // process does not exist anymore; in that case, try to get program name
+      // from probe
+      // note: this fails if the probe contains a wildcard, since the probe id
+      // is not generated per match
+      auto probe_full = resolve_probe(probe_id);
+      if (probe_full.find(',') == std::string::npos &&
+          !has_wildcard(probe_full))
+      {
+        // only find program name for probes that contain one program name,
+        // to avoid incorrect symbol resolutions
+        size_t start = probe_full.find(':') + 1;
+        size_t end = probe_full.find(':', start);
+        pid_exe = probe_full.substr(start, end - start);
+      }
+    }
+    if (user_symbol_cache_type_ == UserSymbolCacheType::per_program)
+    {
+      if (!pid_exe.empty())
+      {
+        // try to resolve symbol directly from program file
+        // this might work when the process does not exist anymore, but cannot
+        // resolve all symbols, e.g. those in a dynamically linked library
+        std::map<uintptr_t, elf_symbol, std::greater<>> &symbol_table =
+            symbol_table_cache_.find(pid_exe) != symbol_table_cache_.end()
+                ? symbol_table_cache_[pid_exe]
+                : (symbol_table_cache_[pid_exe] = get_symbol_table_for_elf(
+                       pid_exe));
+        auto sym = symbol_table.lower_bound(addr);
+        // address has to be either the start of the symbol (for symbols of
+        // length 0) or in [start, end)
+        if (sym != symbol_table.end() &&
+            (addr == sym->second.start ||
+             (addr >= sym->second.start && addr < sym->second.end)))
+        {
+          symbol << sym->second.name;
+          if (show_offset)
+            symbol << "+" << addr - sym->second.start;
+          if (show_module)
+            symbol << " (" << pid_exe << ")";
+          return symbol.str();
+        }
+      }
       if (exe_sym_.find(pid_exe) == exe_sym_.end())
       {
         // not cached, create new ProcSyms cache
@@ -2141,8 +2363,23 @@ std::string BPFtrace::resolve_usym(uintptr_t addr, int pid, bool show_offset, bo
         psyms = exe_sym_[pid_exe].second;
       }
     }
+    else if (user_symbol_cache_type_ == UserSymbolCacheType::per_pid)
+    {
+      // cache user symbols per pid
+      if (pid_sym_.find(pid) == pid_sym_.end())
+      {
+        // not cached, create new ProcSyms cache
+        psyms = bcc_symcache_new(pid, &symopts);
+        pid_sym_[pid] = psyms;
+      }
+      else
+      {
+        psyms = pid_sym_[pid];
+      }
+    }
     else
     {
+      // no user symbol caching, create new bcc cache
       psyms = bcc_symcache_new(pid, &symopts);
     }
   }
@@ -2165,7 +2402,8 @@ std::string BPFtrace::resolve_usym(uintptr_t addr, int pid, bool show_offset, bo
       symbol << " ([unknown])";
   }
 
-  if (psyms && !cache_user_symbols_)
+  if (resolve_user_symbols_ &&
+      user_symbol_cache_type_ == UserSymbolCacheType::none)
     bcc_free_symcache(psyms, pid);
 
   return symbol.str();
