@@ -30,6 +30,8 @@
 
 namespace bpftrace {
 
+const int timeout_ms = 100;
+
 struct symbol
 {
   std::string name;
@@ -116,13 +118,24 @@ public:
   int clear_map(IMap &map);
   int zero_map(IMap &map);
   int print_map(IMap &map, uint32_t top, uint32_t div);
-  std::string get_stack(uint64_t stackidpid, bool ustack, StackType stack_type, int indent=0);
+  std::string get_stack(int64_t stackid,
+                        int pid,
+                        int probe_id,
+                        bool ustack,
+                        StackType stack_type,
+                        int indent = 0);
   std::string resolve_buf(char *buf, size_t size);
   std::string resolve_ksym(uintptr_t addr, bool show_offset=false);
-  std::string resolve_usym(uintptr_t addr, int pid, bool show_offset=false, bool show_module=false);
+  std::string resolve_usym(uintptr_t addr,
+                           int pid,
+                           int probe_id,
+                           bool show_offset = false,
+                           bool show_module = false);
   std::string resolve_inet(int af, const uint8_t* inet) const;
   std::string resolve_uid(uintptr_t addr) const;
-  std::string resolve_timestamp(uint32_t strftime_id, uint64_t nsecs);
+  std::string resolve_timestamp(uint32_t mode,
+                                uint32_t strftime_id,
+                                uint64_t nsecs);
   uint64_t resolve_kname(const std::string &name) const;
   virtual int resolve_uname(const std::string &name,
                             struct symbol *sym,
@@ -143,7 +156,7 @@ public:
   std::optional<int64_t> get_int_literal(const ast::Expression *expr) const;
   std::optional<std::string> get_watchpoint_binary_path() const;
   virtual bool is_traceable_func(const std::string &func_name) const;
-  std::unordered_set<std::string> get_func_modules(
+  virtual std::unordered_set<std::string> get_func_modules(
       const std::string &func_name) const;
   int create_pcaps(void);
   void close_pcaps(void);
@@ -190,7 +203,12 @@ public:
   uint64_t max_type_res_iterations = 0;
   bool demangle_cpp_symbols_ = true;
   bool resolve_user_symbols_ = true;
-  bool cache_user_symbols_ = true;
+  enum class UserSymbolCacheType
+  {
+    per_pid,
+    per_program,
+    none,
+  } user_symbol_cache_type_;
   bool safe_mode_ = true;
   bool has_usdt_ = false;
   bool usdt_file_activation_ = false;
@@ -198,6 +216,9 @@ public:
   uint64_t ast_max_nodes_ = 0; // Maximum AST nodes allowed for fuzzing
   std::optional<StackMode> stack_mode_;
   std::optional<struct timespec> boottime_;
+  std::optional<struct timespec> delta_taitime_;
+  static constexpr uint32_t rb_loss_cnt_key_ = 0;
+  static constexpr uint64_t rb_loss_cnt_val_ = 0;
 
   static void sort_by_key(
       std::vector<SizedType> key_args,
@@ -208,7 +229,6 @@ public:
 
   std::unique_ptr<BTF> btf_;
   std::unordered_set<std::string> btf_set_;
-  std::map<std::string, ProbeArgs> ap_args_;
   std::unique_ptr<ChildProcBase> child_;
   std::unique_ptr<ProcMonBase> procmon_;
   pid_t pid(void) const
@@ -223,7 +243,11 @@ private:
                         BpfBytecode &bytecode,
                         void (*trigger)(void));
   void* ksyms_{nullptr};
+  // note: exe_sym_ is used when layout is same for all instances of program
   std::map<std::string, std::pair<int, void *>> exe_sym_; // exe -> (pid, cache)
+  std::map<int, void *> pid_sym_;                         // pid -> cache
+  std::map<std::string, std::map<uintptr_t, elf_symbol, std::greater<>>>
+      symbol_table_cache_;
   std::vector<std::string> params_;
 
   std::vector<std::unique_ptr<void, void (*)(void *)>> open_perf_buffers_;
@@ -233,14 +257,33 @@ private:
       std::tuple<uint8_t *, uintptr_t> func,
       int pid,
       bool file_activation);
+  int setup_output();
   int setup_perf_events();
-  void poll_perf_events(bool drain = false);
+  int setup_ringbuf();
+  // when the ringbuf feature is available, enable ringbuf for built-ins like
+  // printf, cat.
+  bool is_ringbuf_enabled(void) const
+  {
+    return feature_->has_map_ringbuf();
+  }
+  // when the ringbuf feature is unavailable or built-in skboutput is used,
+  // enable perf_event
+  bool is_perf_event_enabled(void) const
+  {
+    return !feature_->has_map_ringbuf() || resources.needs_perf_event_map;
+  }
+  void teardown_output();
+  void poll_output(bool drain = false);
+  int poll_perf_events();
+  void handle_ringbuf_loss();
   int print_map_hist(IMap &map, uint32_t top, uint32_t div);
   int print_map_stats(IMap &map, uint32_t top, uint32_t div);
   static uint64_t read_address_from_output(std::string output);
   std::vector<uint8_t> find_empty_key(IMap &map, size_t size) const;
   bool has_iter_ = false;
   int epollfd_ = -1;
+  struct ring_buffer *ringbuf_ = nullptr;
+  uint64_t ringbuf_loss_count_ = 0;
 
   std::unordered_map<std::string, std::unique_ptr<Dwarf>> dwarves_;
 };
