@@ -10,6 +10,7 @@
 #include <fcntl.h>
 #include <iostream>
 #include <linux/limits.h>
+#include <optional>
 #include <regex>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -414,9 +415,7 @@ SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
   return stype;
 }
 
-void BTF::resolve_args(const std::string &func,
-                       std::map<std::string, SizedType> &args,
-                       bool ret)
+Struct BTF::resolve_args(const std::string &func, bool ret)
 {
   if (!has_data())
     throw std::runtime_error("BTF data not available");
@@ -449,6 +448,7 @@ void BTF::resolve_args(const std::string &func,
                              "not supported.");
   }
 
+  Struct args(0, false);
   int j = 0;
   for (; j < vlen; j++, p++)
   {
@@ -459,7 +459,11 @@ void BTF::resolve_args(const std::string &func,
     SizedType stype = get_stype(BTFId{ .btf = func_id.btf, .id = p->type });
     stype.funcarg_idx = j;
     stype.is_funcarg = true;
-    args.insert({ str, stype });
+    args.AddField(str, stype, args.size, std::nullopt, false);
+    // kfunc (fentry/fexit) args are stored in a u64 array.
+    // Note that it's ok to represent them by a struct as we will use GEP with
+    // funcarg_idx to access them in codegen.
+    args.size += 8;
   }
 
   if (ret)
@@ -467,8 +471,11 @@ void BTF::resolve_args(const std::string &func,
     SizedType stype = get_stype(BTFId{ .btf = func_id.btf, .id = t->type });
     stype.funcarg_idx = j;
     stype.is_funcarg = true;
-    args.insert({ "$retval", stype });
+    args.AddField("$retval", stype, args.size, std::nullopt, false);
+    // kfunc (fentry/fexit) args (incl. retval) are stored in a u64 array
+    args.size += 8;
   }
+  return args;
 }
 
 std::string BTF::get_all_funcs_from_btf(const BTFObj &btf_obj) const
@@ -714,6 +721,49 @@ std::set<std::string> BTF::get_all_structs() const
   return structs;
 }
 
+std::unordered_set<std::string> BTF::get_all_iters_from_btf(
+    const struct btf *btf) const
+{
+  std::unordered_set<std::string> iter_set;
+  const std::string prefix = "bpf_iter__";
+  // kernel commit 6fcd486b3a0a("bpf: Refactor RCU enforcement in the
+  // verifier.") add 'struct bpf_iter__task__safe_trusted'
+  const std::string suffix___safe_trusted = "__safe_trusted";
+  __s32 id, max = (__s32)type_cnt(btf);
+  for (id = start_id(btf); id <= max; id++)
+  {
+    const struct btf_type *t = btf__type_by_id(btf, id);
+
+    if (!t || !(btf_is_struct(t)))
+      continue;
+
+    const std::string name = btf_str(btf, t->name_off);
+
+    // skip __safe_trusted suffix struct
+    if (suffix___safe_trusted.length() < name.length() &&
+        name.rfind(suffix___safe_trusted) ==
+            (name.length() - suffix___safe_trusted.length()))
+      continue;
+    if (name.size() > prefix.size() &&
+        name.compare(0, prefix.size(), prefix) == 0)
+    {
+      iter_set.insert(name.substr(prefix.size()));
+    }
+  }
+  return iter_set;
+}
+
+std::unordered_set<std::string> BTF::get_all_iters() const
+{
+  std::unordered_set<std::string> iters;
+  for (auto &btf_obj : btf_objects)
+  {
+    auto mod_iters = get_all_iters_from_btf(btf_obj.btf);
+    iters.insert(mod_iters.begin(), mod_iters.end());
+  }
+  return iters;
+}
+
 // Retrieves BTF id of the given function and the FD of the BTF object
 // containing it
 std::pair<int, int> BTF::get_btf_id_fd(const std::string &func,
