@@ -109,6 +109,7 @@ void usage()
   std::cerr << std::endl;
   std::cerr << "TROUBLESHOOTING OPTIONS:" << std::endl;
   std::cerr << "    -v                      verbose messages" << std::endl;
+  std::cerr << "    -vv                     more verbose messages (max 2)" << std::endl;
   std::cerr << "    -d                      (dry run) debug info" << std::endl;
   std::cerr << "    -dd                     (dry run) verbose debug info" << std::endl;
   std::cerr << "    --emit-elf FILE         (dry run) generate ELF file with bpf programs and write to FILE" << std::endl;
@@ -181,7 +182,7 @@ static void info()
   std::cerr << BPFfeature().report();
 }
 
-static std::optional<struct timespec> get_boottime()
+static std::optional<struct timespec> get_delta_with_boottime(int clock_type)
 {
   std::optional<struct timespec> ret = std::nullopt;
   long lowest_delta = std::numeric_limits<long>::max();
@@ -194,13 +195,13 @@ static std::optional<struct timespec> get_boottime()
     struct timespec before, after, boottime;
     long delta;
 
-    if (::clock_gettime(CLOCK_REALTIME, &before))
+    if (::clock_gettime(clock_type, &before))
       continue;
 
     if (::clock_gettime(CLOCK_BOOTTIME, &boottime))
       continue;
 
-    if (::clock_gettime(CLOCK_REALTIME, &after))
+    if (::clock_gettime(clock_type, &after))
       continue;
 
     // There's no way 3 VDSO calls should take more than 1s. We'll
@@ -219,21 +220,21 @@ static std::optional<struct timespec> get_boottime()
     // Lowest delta seen so far, compute boot realtime and store it
     if (delta < lowest_delta)
     {
-      struct timespec boottime_realtime;
+      struct timespec delta_with_boottime;
       long nsec_avg = (before.tv_nsec + after.tv_nsec) / 2;
       if (nsec_avg - boottime.tv_nsec < 0)
       {
-        boottime_realtime.tv_sec = after.tv_sec - boottime.tv_sec - 1;
-        boottime_realtime.tv_nsec = nsec_avg - boottime.tv_nsec + 1e9;
+        delta_with_boottime.tv_sec = after.tv_sec - boottime.tv_sec - 1;
+        delta_with_boottime.tv_nsec = nsec_avg - boottime.tv_nsec + 1e9;
       }
       else
       {
-        boottime_realtime.tv_sec = after.tv_sec - boottime.tv_sec;
-        boottime_realtime.tv_nsec = nsec_avg - boottime.tv_nsec;
+        delta_with_boottime.tv_sec = after.tv_sec - boottime.tv_sec;
+        delta_with_boottime.tv_nsec = nsec_avg - boottime.tv_nsec;
       }
 
       lowest_delta = delta;
-      ret = boottime_realtime;
+      ret = delta_with_boottime;
     }
   }
 
@@ -245,6 +246,16 @@ static std::optional<struct timespec> get_boottime()
   return ret;
 }
 
+static std::optional<struct timespec> get_boottime()
+{
+  return get_delta_with_boottime(CLOCK_REALTIME);
+}
+
+static std::optional<struct timespec> get_delta_taitime()
+{
+  return get_delta_with_boottime(CLOCK_TAI);
+}
+
 [[nodiscard]] static bool parse_env(BPFtrace& bpftrace, bool& verify_llvm_ir)
 {
   if (!get_uint64_env_var("BPFTRACE_STRLEN", bpftrace.strlen_))
@@ -309,21 +320,45 @@ static std::optional<struct timespec> get_boottime()
     bpftrace.cat_bytes_max_ = proposed;
   }
 
-  if (!get_bool_env_var("BPFTRACE_NO_USER_SYMBOLS",
-                        bpftrace.resolve_user_symbols_,
-                        true))
-    return false;
-
-  if (!get_bool_env_var("BPFTRACE_CACHE_USER_SYMBOLS",
-                        bpftrace.cache_user_symbols_))
-    return false;
-
-  if (!std::getenv("BPFTRACE_CACHE_USER_SYMBOLS"))
+  // by default, cache user symbols per program if ASLR is disabled on system
+  // or `-c` option is given, otherwise cache per PID
+  auto default_usym_cache_type =
+      (!bpftrace.cmd_.empty() || !bpftrace.is_aslr_enabled(-1))
+          ? BPFtrace::UserSymbolCacheType::per_program
+          : BPFtrace::UserSymbolCacheType::per_pid;
+  if (const char* env_p = std::getenv("BPFTRACE_CACHE_USER_SYMBOLS"))
   {
-    // enable user symbol cache if ASLR is disabled on system or `-c` option is
-    // given
-    bpftrace.cache_user_symbols_ = !bpftrace.cmd_.empty() ||
-                                   !bpftrace.is_aslr_enabled(-1);
+    std::string s(env_p);
+    // Note: options 0 and 1 are for compatibility with older versions of
+    // bpftrace
+    if (s == "PER_PID")
+    {
+      bpftrace.user_symbol_cache_type_ = BPFtrace::UserSymbolCacheType::per_pid;
+    }
+    else if (s == "PER_PROGRAM")
+    {
+      bpftrace.user_symbol_cache_type_ =
+          BPFtrace::UserSymbolCacheType::per_program;
+    }
+    else if (s == "1")
+    {
+      bpftrace.user_symbol_cache_type_ = default_usym_cache_type;
+    }
+    else if (s == "NONE" || s == "0")
+    {
+      bpftrace.user_symbol_cache_type_ = BPFtrace::UserSymbolCacheType::none;
+    }
+    else
+    {
+      LOG(ERROR)
+          << "Env var 'BPFTRACE_CACHE_USER_SYMBOLS' did not contain a valid "
+             "value: valid values are PER_PID, PER_PROGRAM, and NONE.";
+      return false;
+    }
+  }
+  else
+  {
+    bpftrace.user_symbol_cache_type_ = default_usym_cache_type;
   }
 
   uint64_t node_max = std::numeric_limits<uint64_t>::max();
@@ -384,7 +419,7 @@ static std::optional<struct timespec> get_boottime()
     struct utsname utsname;
     uname(&utsname);
     std::string ksrc, kobj;
-    auto kdirs = get_kernel_dirs(utsname, !bpftrace.feature_->has_btf());
+    auto kdirs = get_kernel_dirs(utsname);
     ksrc = std::get<0>(kdirs);
     kobj = std::get<1>(kdirs);
 
@@ -771,6 +806,7 @@ int main(int argc, char* argv[])
   bpftrace.safe_mode_ = args.safe_mode;
   bpftrace.helper_check_level_ = args.helper_check_level;
   bpftrace.boottime_ = get_boottime();
+  bpftrace.delta_taitime_ = get_delta_taitime();
 
   if (!args.pid_str.empty())
   {
