@@ -39,8 +39,9 @@ static int add_symbol(const char* symname,
  */
 std::set<std::string> ProbeMatcher::get_matches_in_stream(
     const std::string& search_input,
-    bool ignore_trailing_module,
     std::istream& symbol_stream,
+    bool ignore_trailing_module,
+    bool demangle_symbols,
     const char delim)
 {
   bool start_wildcard, end_wildcard;
@@ -57,16 +58,33 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
 
     if (!wildcard_match(line, tokens, start_wildcard, end_wildcard))
     {
-      auto fun_line = line;
-      auto prefix = fun_line.find(':') != std::string::npos
-                        ? erase_prefix(fun_line) + ":"
-                        : "";
-      if (symbol_has_cpp_mangled_signature(fun_line))
+      if (demangle_symbols)
       {
-        char* demangled_name = cxxdemangle(fun_line.c_str());
-        if (demangled_name)
+        auto fun_line = line;
+        auto prefix = fun_line.find(':') != std::string::npos
+                          ? erase_prefix(fun_line) + ":"
+                          : "";
+        if (symbol_has_cpp_mangled_signature(fun_line))
         {
-          if (!wildcard_match(prefix + demangled_name, tokens, true, true))
+          char* demangled_name = cxxdemangle(fun_line.c_str());
+          if (!demangled_name)
+            continue;
+
+          // Match against the demanled name.
+          // Since demangled_name contains function arguments, we need to remove
+          // them unless the user specified '(' in the search input (i.e. wants
+          // to match against the arguments explicitly).
+          std::string match_line = prefix + demangled_name;
+          if (std::all_of(tokens.begin(),
+                          tokens.end(),
+                          [&](const std::string& token) {
+                            return token.find("(") == std::string::npos;
+                          }))
+          {
+            match_line = match_line.substr(0, match_line.find_last_of("("));
+          }
+
+          if (!wildcard_match(match_line, tokens, start_wildcard, end_wildcard))
           {
             free(demangled_name);
           }
@@ -89,11 +107,6 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
   return matches;
 }
 
-std::unique_ptr<std::istream> ProbeMatcher::get_iter_symbols(void) const
-{
-  return std::make_unique<std::istringstream>("task\ntask_file\ntask_vma");
-}
-
 /*
  * Get matches of search_input (containing a wildcard) for a given probe_type.
  * probe_type determines where to take the candidate matches from.
@@ -102,7 +115,8 @@ std::unique_ptr<std::istream> ProbeMatcher::get_iter_symbols(void) const
 std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     const ProbeType& probe_type,
     const std::string& target,
-    const std::string& search_input)
+    const std::string& search_input,
+    bool demangle_symbols)
 {
   std::unique_ptr<std::istream> symbol_stream;
   bool ignore_trailing_module = false;
@@ -129,6 +143,11 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
       symbol_stream = get_symbols_from_file_safe(tracefs::available_events());
       break;
     }
+    case ProbeType::rawtracepoint: {
+      symbol_stream = get_symbols_from_file_safe(tracefs::available_events());
+      symbol_stream = adjust_rawtracepoint(*symbol_stream);
+      break;
+    }
     case ProbeType::usdt:
     {
       symbol_stream = get_symbols_from_usdt(bpftrace_->pid(), target);
@@ -162,7 +181,30 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     }
     case ProbeType::iter:
     {
-      symbol_stream = get_iter_symbols();
+      if (!bpftrace_->has_btf_data())
+        break;
+
+      std::string ret;
+      auto iters = bpftrace_->btf_->get_all_iters();
+      for (auto& iter : iters)
+      {
+        // second check
+        if (bpftrace_->feature_->has_iter(iter))
+          ret += iter + "\n";
+        else
+          LOG(WARNING) << "The kernel contains bpf_iter__" << iter
+                       << " struct but does not support loading an iterator"
+                          " program against it. Please report this bug.";
+      }
+      symbol_stream = std::make_unique<std::istringstream>(ret);
+      break;
+    }
+    case ProbeType::interval:
+    case ProbeType::profile: {
+      std::string ret;
+      for (auto& unit : TIME_UNITS)
+        ret += unit + ":\n";
+      symbol_stream = std::make_unique<std::istringstream>(ret);
       break;
     }
     default:
@@ -170,9 +212,8 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
   }
 
   if (symbol_stream)
-    return get_matches_in_stream(search_input,
-                                 ignore_trailing_module,
-                                 *symbol_stream);
+    return get_matches_in_stream(
+        search_input, *symbol_stream, ignore_trailing_module, demangle_symbols);
   else
     return {};
 }
@@ -191,7 +232,7 @@ std::set<std::string> ProbeMatcher::get_matches_in_set(
     stream_in.append(str + "$");
 
   std::istringstream stream(stream_in);
-  return get_matches_in_stream(search_input, false, stream, '$');
+  return get_matches_in_stream(search_input, stream, false, false, '$');
 }
 
 std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_file(
@@ -285,6 +326,8 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_usdt(
 
   if (pid > 0)
     usdt_probes = USDTHelper::probes_for_pid(pid);
+  else if (target == "*")
+    usdt_probes = USDTHelper::probes_for_all_pids();
   else if (!target.empty())
   {
     std::vector<std::string> real_paths;
@@ -318,7 +361,11 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_list(
 {
   std::string symbols;
   for (auto& probe : probes_list)
+  {
     symbols += probe.path + ":\n";
+    if (!probe.alias.empty())
+      symbols += probe.alias + ":\n";
+  }
   return std::make_unique<std::istringstream>(symbols);
 }
 
@@ -407,27 +454,25 @@ FuncParamLists ProbeMatcher::get_tracepoints_params(
 FuncParamLists ProbeMatcher::get_iters_params(
     const std::set<std::string>& iters)
 {
+  const std::string prefix = "vmlinux:bpf_iter_";
   FuncParamLists params;
+  std::set<std::string> funcs;
 
   for (auto& iter : iters)
-  {
-    if (iter == "task")
-    {
-      params[iter].push_back("struct task_struct * task");
-    }
-    else if (iter == "task_file")
-    {
-      params[iter].push_back("struct task_struct * task");
-      params[iter].push_back("int fd");
-      params[iter].push_back("struct file * file");
-    }
-    else if (iter == "task_vma")
-    {
-      params[iter].push_back("struct task_struct * task");
-      params[iter].push_back("struct vm_area_struct * vma");
-    }
-  }
+    funcs.insert(prefix + iter);
 
+  params = bpftrace_->btf_->get_params(funcs);
+  for (auto func : funcs)
+  {
+    // delete `int retval`
+    params[func].pop_back();
+    // delete `struct bpf_iter_meta * meta`
+    params[func].erase(params[func].begin());
+    // restore key value
+    auto param = params.extract(func);
+    param.key() = func.substr(prefix.size());
+    params.insert(std::move(param));
+  }
   return params;
 }
 
@@ -479,7 +524,23 @@ void ProbeMatcher::list_probes(ast::Program* prog)
 
       for (auto& match : matches)
       {
-        std::cout << probetypeName(probe_type) << ":" << match << std::endl;
+        std::string match_print = match;
+        if (ap->lang == "cpp")
+        {
+          std::string target = erase_prefix(match_print);
+          char* demangled_name = cxxdemangle(match_print.c_str());
+
+          // demangled name may contain symbols not accepted by the attach point
+          // parser, so surround it with quotes to make the entry directly
+          // usable as an attach point
+          auto func = demangled_name ? "\"" + std::string(demangled_name) + "\""
+                                     : match_print;
+
+          match_print = target + ":" + ap->lang + ":" + func;
+        }
+
+        std::cout << probetypeName(probe_type) << ":" << match_print
+                  << std::endl;
         if (bt_verbose)
         {
           for (auto& param : param_lists[match])
@@ -499,7 +560,7 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
     case ProbeType::kprobe:
     case ProbeType::kretprobe:
     case ProbeType::iter:
-    {
+    case ProbeType::rawtracepoint: {
       search_input = attach_point.func;
       break;
     }
@@ -509,11 +570,8 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
     case ProbeType::watchpoint:
     case ProbeType::asyncwatchpoint:
     case ProbeType::tracepoint:
-    case ProbeType::hardware:
-    case ProbeType::software:
     case ProbeType::kfunc:
-    case ProbeType::kretfunc:
-    case ProbeType::rawtracepoint: {
+    case ProbeType::kretfunc: {
       // Do not expand "target:" as that would match all functions in target.
       // This may occur when an absolute address is given instead of a function.
       if (attach_point.func.empty())
@@ -522,6 +580,13 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
       search_input = attach_point.target + ":" + attach_point.func;
       break;
     }
+    case ProbeType::hardware:
+    case ProbeType::software:
+    case ProbeType::profile:
+    case ProbeType::interval: {
+      search_input = attach_point.target + ":";
+      break;
+    }
     case ProbeType::usdt:
     {
       auto target = attach_point.target;
@@ -546,23 +611,19 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
       throw WildcardException(
           "Wildcard matches aren't available on probe type '" +
           attach_point.provider + "'");
-    case ProbeType::profile:
-    case ProbeType::interval:
-      // Wildcard matches are not supported on these probe types, however
-      // expansion can still happen when the probe builtin is used
-      return { "" };
   }
 
   return get_matches_for_probetype(probetype(attach_point.provider),
                                    attach_point.target,
-                                   search_input);
+                                   search_input,
+                                   attach_point.lang == "cpp");
 }
 
 std::set<std::string> ProbeMatcher::expand_probetype_kernel(
     const std::string& probe_type)
 {
   if (has_wildcard(probe_type))
-    return get_matches_in_stream(probe_type, false, *kernel_probe_list());
+    return get_matches_in_stream(probe_type, *kernel_probe_list());
   else
     return { probe_type };
 }
@@ -571,7 +632,7 @@ std::set<std::string> ProbeMatcher::expand_probetype_userspace(
     const std::string& probe_type)
 {
   if (has_wildcard(probe_type))
-    return get_matches_in_stream(probe_type, false, *userspace_probe_list());
+    return get_matches_in_stream(probe_type, *userspace_probe_list());
   else
     return { probe_type };
 }
@@ -609,4 +670,20 @@ std::unique_ptr<std::istream> ProbeMatcher::adjust_kernel_modules(
   return new_list;
 }
 
+std::unique_ptr<std::istream> ProbeMatcher::adjust_rawtracepoint(
+    std::istream& symbol_list) const
+{
+  auto new_list = std::make_unique<std::stringstream>();
+  std::string line;
+  while (std::getline(symbol_list, line, '\n'))
+  {
+    if ((line.find("syscalls:sys_enter_") != std::string::npos) ||
+        (line.find("syscalls:sys_exit_") != std::string::npos))
+      continue;
+    erase_prefix(line);
+    *new_list << line << "\n";
+  }
+  return new_list;
+}
+
 } // namespace bpftrace
