@@ -119,9 +119,10 @@ bool SizedType::operator==(const SizedType &t) const
 
 bool SizedType::IsByteArray() const
 {
-  return type == Type::string || type == Type::usym || type == Type::inet ||
-         type == Type::buffer || type == Type::timestamp ||
-         type == Type::mac_address || type == Type::cgroup_path;
+  return type == Type::string || type == Type::usym || type == Type::ustack ||
+         type == Type::inet || type == Type::buffer ||
+         type == Type::timestamp || type == Type::mac_address ||
+         type == Type::cgroup_path;
 }
 
 bool SizedType::IsAggregate() const
@@ -185,6 +186,7 @@ std::string typestr(Type t)
     case Type::mac_address: return "mac_address"; break;
     case Type::cgroup_path: return "cgroup_path"; break;
     case Type::strerror: return "strerror"; break;
+    case Type::timestamp_mode: return "timestamp mode"; break;
       // clang-format on
   }
 
@@ -272,7 +274,7 @@ SizedType CreateInteger(size_t bits, bool is_signed)
   // passes infer the exact size.
   assert(bits == 0 || bits == 1 || bits == 8 || bits == 16 || bits == 32 ||
          bits == 64);
-  auto t = SizedType(Type::integer, bits / 8, is_signed);
+  auto t = SizedType(Type::integer, 0, is_signed);
   t.size_bits_ = bits;
   return t;
 }
@@ -351,7 +353,6 @@ SizedType CreateArray(size_t num_elements, const SizedType &element_type)
 {
   size_t size = num_elements * element_type.GetSize();
   auto ty = SizedType(Type::array, size);
-  ty.num_elements_ = num_elements;
   ty.element_type_ = std::make_shared<SizedType>(element_type);
   return ty;
 }
@@ -375,7 +376,7 @@ SizedType CreateRecord(const std::string &name, std::weak_ptr<Struct> record)
 
 SizedType CreateStack(bool kernel, StackType stack)
 {
-  auto st = SizedType(kernel ? Type::kstack : Type::ustack, 8);
+  auto st = SizedType(kernel ? Type::kstack : Type::ustack, kernel ? 8 : 16);
   st.stack_type = stack;
   return st;
 }
@@ -439,7 +440,7 @@ SizedType CreateHist()
 
 SizedType CreateUSym()
 {
-  return SizedType(Type::usym, 16);
+  return SizedType(Type::usym, 24);
 }
 
 SizedType CreateKSym()
@@ -481,6 +482,11 @@ SizedType CreateStrerror()
   return SizedType(Type::strerror, 8);
 }
 
+SizedType CreateTimestampMode()
+{
+  return SizedType(Type::timestamp_mode, 0);
+}
+
 bool SizedType::IsSigned(void) const
 {
   return is_signed_;
@@ -516,9 +522,9 @@ void SizedType::DumpStructure(std::ostream &os)
   return inner_struct_.lock()->Dump(os);
 }
 
-ssize_t SizedType::GetAlignment() const
+ssize_t SizedType::GetInTupleAlignment() const
 {
-  if (IsStringTy())
+  if (IsByteArray())
     return 1;
 
   if (IsTupleTy() || IsRecordTy())
@@ -527,8 +533,8 @@ ssize_t SizedType::GetAlignment() const
   if (GetSize() <= 2)
     return GetSize();
   else if (IsArrayTy())
-    return element_type_->GetAlignment();
-  else if (IsByteArray() || GetSize() <= 4)
+    return element_type_->GetInTupleAlignment();
+  else if (GetSize() <= 4)
     return 4;
   else
     return 8;
@@ -629,6 +635,7 @@ size_t hash<bpftrace::SizedType>::operator()(
     case bpftrace::Type::mac_address:
     case bpftrace::Type::cgroup_path:
     case bpftrace::Type::strerror:
+    case bpftrace::Type::timestamp_mode:
       break;
   }
 
