@@ -5,7 +5,9 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <fstream>
+#include <gelf.h>
 #include <glob.h>
+#include <libelf.h>
 #include <limits>
 #include <link.h>
 #include <map>
@@ -395,6 +397,10 @@ std::vector<std::string> get_kernel_cflags(const char *uname_machine,
   } else if (!strncmp(uname_machine, "aarch64", 7)) {
     arch = "arm64";
   }
+  else if (!strncmp(uname_machine, "loongarch", 9))
+  {
+    arch = "loongarch";
+  }
 
   // If ARCH env is defined, use it over uname
   archenv = getenv("ARCH");
@@ -762,7 +768,14 @@ bool is_compile_time_func(const std::string &func_name)
                      [&](const auto &cand) { return func_name == cand; });
 }
 
-std::string exec_system(const char* cmd)
+bool is_supported_lang(const std::string &lang)
+{
+  return std::any_of(UPROBE_LANGS.begin(),
+                     UPROBE_LANGS.end(),
+                     [&](const auto &cand) { return lang == cand; });
+}
+
+std::string exec_system(const char *cmd)
 {
   std::array<char, 128> buffer;
   std::string result;
@@ -1276,25 +1289,6 @@ std::string strip_symbol_module(const std::string &symbol)
   return idx != std::string::npos ? symbol.substr(0, idx) : symbol;
 }
 
-int get_kernel_ptr_width()
-{
-  // We can't assume that sizeof(void*) in bpftrace is the same as the kernel
-  // pointer size (bpftrace can be compiled as a 32-bit binary and run on a
-  // 64-bit kernel), so we guess based on the machine field of struct utsname.
-  // Note that the uname() syscall can return different values for compat mode
-  // processes (e.g. "armv8l" instead of "aarch64"; see COMPAT_UTS_MACHINE), so
-  // make sure this is taken into account.
-  struct utsname utsname;
-  if (uname(&utsname) != 0)
-    LOG(FATAL) << "uname failed: " << strerror(errno);
-
-  const char *machine = utsname.machine;
-  if (!strncmp(machine, "armv7", 5))
-    return 32;
-
-  return 64;
-}
-
 std::pair<std::string, std::string> split_symbol_module(
     const std::string &symbol)
 {
@@ -1335,4 +1329,61 @@ std::tuple<std::string, std::string, std::string> split_addrrange_symbol_module(
                          symbol.length() - idx2 - strlen(" []")) };
 }
 
+std::map<uintptr_t, elf_symbol, std::greater<>> get_symbol_table_for_elf(
+    const std::string &elf_file)
+{
+  std::map<uintptr_t, elf_symbol, std::greater<>> symbol_table;
+
+  bcc_elf_symcb sym_resolve_callback = [](const char *name,
+                                          uint64_t start,
+                                          uint64_t length,
+                                          void *payload) {
+    auto *symbol_table =
+        static_cast<std::map<uintptr_t, elf_symbol, std::greater<>> *>(payload);
+    symbol_table->insert({ start,
+                           { .name = std::string(name),
+                             .start = start,
+                             .end = start + length } });
+    return 0;
+  };
+  struct bcc_symbol_option option;
+  memset(&option, 0, sizeof(option));
+  option.use_symbol_type = BCC_SYM_ALL_TYPES;
+  bcc_elf_foreach_sym(
+      elf_file.c_str(), sym_resolve_callback, &option, &symbol_table);
+
+  return symbol_table;
+}
+
+std::vector<int> get_pids_for_program(const std::string &program)
+{
+  std::vector<int> pids;
+  auto program_abs = std_filesystem::canonical(program);
+  for (const auto &process : std_filesystem::directory_iterator("/proc"))
+  {
+    std::string filename = process.path().filename().string();
+    if (!std::all_of(filename.begin(), filename.end(), ::isdigit))
+      continue;
+    std::error_code ec;
+    std_filesystem::path pid_program = std_filesystem::read_symlink(
+        process.path() / "exe", ec);
+    if (!ec && program_abs == pid_program)
+      pids.emplace_back(std::stoi(filename));
+  }
+  return pids;
+}
+
+std::vector<int> get_all_running_pids()
+{
+  std::vector<int> pids;
+  for (const auto &process : std_filesystem::directory_iterator("/proc"))
+  {
+    std::string filename = process.path().filename().string();
+    if (!std::all_of(filename.begin(), filename.end(), ::isdigit))
+      continue;
+    pids.emplace_back(std::stoi(filename));
+  }
+  return pids;
+}
+
 } // namespace bpftrace
