@@ -4,6 +4,7 @@
 #include <cstring>
 #include <exception>
 #include <iostream>
+#include <map>
 #include <optional>
 #include <sstream>
 #include <string>
@@ -154,6 +155,8 @@ static std::vector<std::string> UNSAFE_BUILTIN_FUNCS = {
 
 static std::vector<std::string> COMPILE_TIME_FUNCS = { "cgroupid" };
 
+static std::vector<std::string> UPROBE_LANGS = { "cpp" };
+
 bool get_uint64_env_var(const ::std::string &str, uint64_t &dest);
 bool get_bool_env_var(const ::std::string &str, bool &dest, bool neg = false);
 std::string get_pid_exe(pid_t pid);
@@ -175,7 +178,7 @@ std::vector<int> get_possible_cpus();
 bool is_dir(const std::string &path);
 std::tuple<std::string, std::string> get_kernel_dirs(
     const struct utsname &utsname,
-    bool unpack_kheaders);
+    bool unpack_kheaders = true);
 std::vector<std::string> get_kernel_cflags(const char *uname_machine,
                                            const std::string &ksrc,
                                            const std::string &kobj,
@@ -190,6 +193,7 @@ FuncsModulesMap get_traceable_funcs();
 const std::string &is_deprecated(const std::string &str);
 bool is_unsafe_func(const std::string &func_name);
 bool is_compile_time_func(const std::string &func_name);
+bool is_supported_lang(const std::string &lang);
 std::string exec_system(const char *cmd);
 std::vector<std::string> resolve_binary_path(const std::string &cmd);
 std::vector<std::string> resolve_binary_path(const std::string &cmd, int pid);
@@ -212,6 +216,21 @@ std::pair<std::string, std::string> split_symbol_module(
 std::tuple<std::string, std::string, std::string> split_addrrange_symbol_module(
     const std::string &symbol);
 
+struct elf_symbol
+{
+  std::string name;
+  uintptr_t start;
+  uintptr_t end;
+};
+// Get all symbols from an ELF module together with their address ranges in
+// the form of a map sorted by start address.
+// Note: the map uses std::greater as comparator to allow resolving of an
+// address inside a range using std::map::lower_bound.
+std::map<uintptr_t, elf_symbol, std::greater<>> get_symbol_table_for_elf(
+    const std::string &elf_file);
+std::vector<int> get_pids_for_program(const std::string &program);
+std::vector<int> get_all_running_pids();
+
 // Generate object file section name for a given probe
 inline std::string get_section_name_for_probe(
     const std::string &probe_name,
@@ -294,7 +313,4 @@ inline void hash_combine(std::size_t &seed, const T &value)
   seed ^= hasher(value) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
 }
 
-// Returns the width in bits of kernel pointers.
-int get_kernel_ptr_width();
-
 } // namespace bpftrace
