@@ -1,7 +1,10 @@
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
+#include <fstream>
+#include <iostream>
 #include <sstream>
 #include <string>
+#include <system_error>
 
 #include <fcntl.h>
 #include <signal.h>
@@ -12,6 +15,16 @@
 #include "childhelper.h"
 #include "utils.h"
 
+#if __has_include(<filesystem>)
+#include <filesystem>
+namespace std_filesystem = std::filesystem;
+#elif __has_include(<experimental/filesystem>)
+#include <experimental/filesystem>
+namespace std_filesystem = std::experimental::filesystem;
+#else
+#error "neither <filesystem> nor <experimental/filesystem> are present"
+#endif
+
 namespace bpftrace {
 namespace test {
 namespace child {
@@ -204,6 +217,62 @@ TEST(childproc, ptrace_child_term_before_execve)
   EXPECT_EQ(child->term_signal(), 15);
 }
 
+TEST(childproc, multi_exec_match)
+{
+  std::error_code ec;
+
+  // Create directory for test
+  std::string tmpdir = "/tmp/bpftrace-test-child-XXXXXX";
+  ASSERT_NE(::mkdtemp(&tmpdir[0]), nullptr);
+
+  // Create fixture directories
+  const auto path = std_filesystem::path(tmpdir);
+  const auto usr_bin = path / "usr" / "bin";
+  ASSERT_TRUE(std_filesystem::create_directories(usr_bin, ec));
+  ASSERT_FALSE(ec);
+
+  // Create symbolic link: bin -> usr/bin
+  const auto symlink_bin = path / "bin";
+  std_filesystem::create_directory_symlink(usr_bin, symlink_bin, ec);
+  ASSERT_FALSE(ec);
+
+  // Copy a 'mysleep' binary and add x permission
+  const auto binary = usr_bin / "mysleep";
+  {
+    std::ifstream src;
+    std::ofstream dst;
+
+    src.open("/bin/sleep", std::ios::in | std::ios::binary);
+    dst.open(binary, std::ios::out | std::ios::binary);
+    dst << src.rdbuf();
+    src.close();
+    dst.close();
+
+    EXPECT_EQ(::chmod(binary.c_str(), 0755), 0);
+  }
+
+  // Set ENV
+  auto old_path = ::getenv("PATH");
+  auto new_path = usr_bin.native(); // copy
+  new_path += ":";
+  new_path += symlink_bin.c_str();
+  EXPECT_EQ(::setenv("PATH", new_path.c_str(), 1), 0);
+
+  // 'mysleep' will match /bin/mysleep and /usr/bin/mysleep, but they are
+  // actually the same file.
+  auto child = getChild("mysleep 5");
+
+  child->run();
+  child->terminate();
+  wait_for(child.get(), 100);
+  EXPECT_FALSE(child->is_alive());
+  EXPECT_EQ(child->term_signal(), SIGTERM);
+
+  // Cleanup
+  EXPECT_EQ(::setenv("PATH", old_path, 1), 0);
+  EXPECT_GT(std_filesystem::remove_all(tmpdir), 0);
+}
+
 } // namespace child
 } // namespace test
 } // namespace bpftrace
