@@ -10,11 +10,7 @@ namespace codegen {
 
 TEST(codegen, args_multiple_tracepoints_category_wild)
 {
-  auto bpftrace = get_mock_bpftrace();
-
-  test(*bpftrace,
-       "tracepoint:sched*:sched_* { @[args->common_field] = count(); }",
-       NAME);
+  test("tracepoint:sched*:sched_* { @[args.common_field] = count(); }", NAME);
 }
 
 } // namespace codegen
