@@ -11,17 +11,27 @@ namespace parser {
 
 using Printer = ast::Printer;
 
-void test_parse_failure(BPFtrace &bpftrace, const std::string &input)
+void test_parse_failure(BPFtrace &bpftrace,
+                        const std::string &input,
+                        std::string_view expected_error = {})
 {
   std::stringstream out;
   Driver driver(bpftrace, out);
   ASSERT_EQ(driver.parse_str(input), 1);
+
+  if (expected_error.data())
+  {
+    if (!expected_error.empty() && expected_error[0] == '\n')
+      expected_error.remove_prefix(1); // Remove initial '\n'
+    EXPECT_EQ(out.str(), expected_error);
+  }
 }
 
-void test_parse_failure(const std::string &input)
+void test_parse_failure(const std::string &input,
+                        std::string_view expected_error = {})
 {
   BPFtrace bpftrace;
-  test_parse_failure(bpftrace, input);
+  test_parse_failure(bpftrace, input, expected_error);
 }
 
 void test(BPFtrace &bpftrace,
@@ -960,11 +970,6 @@ TEST(Parser, call_unknown_function)
 TEST(Parser, call_builtin)
 {
   // Builtins should not be usable as function
-  test_parse_failure("k:f { nsecs(); }");
-  test_parse_failure("k:f { nsecs  (); }");
-  test_parse_failure("k:f { nsecs(\"abc\"); }");
-  test_parse_failure("k:f { nsecs(123); }");
-
   test_parse_failure("k:f { probe(\"blah\"); }");
   test_parse_failure("k:f { probe(); }");
   test_parse_failure("k:f { probe(123); }");
@@ -1020,6 +1025,17 @@ TEST(Parser, uprobe)
        " uprobe:/my/program:A::f\n"
        "  int: 1\n");
 
+  // Language prefix
+  test("uprobe:/my/program:cpp:func { 1; }",
+       "Program\n"
+       " uprobe:/my/program:cpp:func\n"
+       "  int: 1\n");
+
+  test("uprobe:/my/dir+/program:1234abc { 1; }",
+       "Program\n"
+       " uprobe:/my/dir+/program:1234abc\n"
+       "  int: 1\n");
+
   test_parse_failure("uprobe:f { 1 }");
   test_parse_failure("uprobe { 1 }");
   test_parse_failure("uprobe:/my/program*:0x1234 { 1 }");
@@ -1971,6 +1987,18 @@ TEST(Parser, invalid_provider)
   test_parse_failure("asdf:xyz:www { }");
 }
 
+TEST(Parser, non_fatal_errors)
+{
+  // The non-fatal error from parsing "Stream" as an integer should not be
+  // displayed
+  test_parse_failure("uprobe:asdf:Stream {} tracepoint:only_one_arg {}",
+                     R"(
+stdin:1:22-46: ERROR: tracepoint probe type requires 2 arguments
+uprobe:asdf:Stream {} tracepoint:only_one_arg {}
+                     ~~~~~~~~~~~~~~~~~~~~~~~~
+)");
+}
+
 } // namespace parser
 } // namespace test
 } // namespace bpftrace
