@@ -25,8 +25,9 @@ TestStruct = namedtuple(
         'prog',
         'expect',
         'timeout',
-        'before',
+        'befores',
         'after',
+        'cleanup',
         'suite',
         'kernel_min',
         'kernel_max',
@@ -101,8 +102,9 @@ class TestParser(object):
         prog = ''
         expect = ''
         timeout = ''
-        before = ''
+        befores = []
         after = ''
+        cleanup = ''
         kernel_min = ''
         kernel_max = ''
         requirement = []
@@ -128,9 +130,11 @@ class TestParser(object):
             elif item_name == 'TIMEOUT':
                 timeout = int(line.strip(' '))
             elif item_name == 'BEFORE':
-                before = line
+                befores.append(line)
             elif item_name == 'AFTER':
                 after = line
+            elif item_name == 'CLEANUP':
+                cleanup = line
             elif item_name == 'MIN_KERNEL':
                 kernel_min = line
             elif item_name == 'MAX_KERNEL':
@@ -160,6 +164,8 @@ class TestParser(object):
                     "aot",
                     "kprobe_multi",
                     "skboutput",
+                    "get_tai_ns",
+                    "get_func_ip",
                 }
 
                 for f in line.split(" "):
@@ -194,8 +200,9 @@ class TestParser(object):
             prog,
             expect,
             timeout,
-            before,
+            befores,
             after,
+            cleanup,
             test_suite,
             kernel_min,
             kernel_max,
